@@ -167,6 +167,7 @@ void menu_parser_define_macro(WMenuParser parser)
 		return;
 	}
 	macro = wmalloc(sizeof(*macro));
+	memset(arg_name, 0, MAX_MACRO_ARG_COUNT * sizeof(char *));
 
 	/* Isolate name of macro */
 	idx = 0;
@@ -279,7 +280,8 @@ WParserMacro *menu_parser_find_macro(WMenuParser parser, const char *name)
 			continue;
 
 		return macro;
-	check_next_macro: ;
+	check_next_macro:
+		;
 	}
 	return NULL;
 }
@@ -391,6 +393,8 @@ void menu_parser_expand_macro(WMenuParser parser, WParserMacro *macro)
 	unsigned int size;
 	int i, space_left;
 
+	memset(arg_value, 0, MAX_MACRO_ARG_COUNT * sizeof(char *));
+
 	/* Skip the name of the macro, this was not done by caller */
 	for (i = 0; macro->name[i] != '\0'; i++)
 		parser->rd++;
@@ -692,7 +696,7 @@ static void w_create_macro(WMenuParser parser, const char *name, WParserMacroFun
 	WParserMacro *macro;
 
 	macro = wmalloc(sizeof(*macro));
-	strcpy(macro->name, name);
+	strncpy(macro->name, name, sizeof(macro->name) - 1);
 	macro->function = handler;
 	macro->arg_count = -1;
 	macro->next = parser->macros;
