@@ -375,8 +375,6 @@ void WMSetBrowserTitled(WMBrowser * bPtr, Bool flag)
 	if (bPtr->flags.isTitled == flag)
 		return;
 
-	columnX = 0;
-
 	if (!bPtr->flags.isTitled) {
 		columnY = TITLE_SPACING + bPtr->titleHeight;
 
@@ -434,6 +432,9 @@ static void willResizeBrowser(W_ViewDelegate * self, WMView * view, unsigned int
 	int colX, colY;
 	int i;
 
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) self;
+
 	assert(*width > 0);
 	assert(*height > 0);
 
@@ -475,19 +476,24 @@ static void paintItem(WMList * lPtr, int index, Drawable d, char *text, int stat
 	Display *display = scr->display;
 	WMFont *font = ((state & WLDSIsBranch) ? scr->boldFont : scr->normalFont);
 	WMColor *backColor = ((state & WLDSSelected) ? scr->white : view->backColor);
-	int width, height, x, y, textLen;
+	int width, height, x, y;
+
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) index;
 
 	width = rect->size.width;
 	height = rect->size.height;
 	x = rect->pos.x;
 	y = rect->pos.y;
-	textLen = strlen(text);
 
 	XFillRectangle(display, d, WMColorGC(backColor), x, y, width, height);
 
 	if (text) {
+		int widthC, textLen;
+
 		/* Avoid overlaping... */
-		int widthC = (state & WLDSIsBranch) ? width - 20 : width - 8;
+		widthC = (state & WLDSIsBranch) ? width - 20 : width - 8;
+		textLen = strlen(text);
 		if (WMWidthOfString(font, text, textLen) > widthC) {
 			char *textBuf = createTruncatedString(font, text, &textLen, widthC);
 			W_PaintText(view, d, font, x + 4, y, widthC, WALeft, scr->black, False, textBuf, textLen);
@@ -559,7 +565,7 @@ static void scrollCallback(WMWidget * scroller, void *self)
 
 			floatValue = (floatValue * value) / value;
 
-			newFirst = rint(floatValue * (float)(bPtr->columnCount - bPtr->maxVisibleColumns));
+			newFirst = rint(floatValue * (double)(bPtr->columnCount - bPtr->maxVisibleColumns));
 
 			if (bPtr->firstVisibleColumn != newFirst)
 				scrollToColumn(bPtr, newFirst, False);
@@ -613,11 +619,12 @@ void WMSetBrowserHasScroller(WMBrowser * bPtr, int hasScroller)
 	bPtr->flags.hasScroller = hasScroller;
 }
 
-char *WMSetBrowserPath(WMBrowser * bPtr, char *path)
+const char *WMSetBrowserPath(WMBrowser * bPtr, const char *path)
 {
 	int i;
 	char *str;
-	char *tmp, *retPtr = NULL;
+	char *tmp;
+	const char *retPtr = NULL;
 	int item;
 	WMListItem *listItem;
 
@@ -775,10 +782,7 @@ WMArray *WMGetBrowserPaths(WMBrowser * bPtr)
 		path = wmalloc(slen);
 		/* ignore first `/' */
 		for (i = 0; i <= column; i++) {
-			if (wstrlcat(path, bPtr->pathSeparator, slen) >= slen) {
-				wfree(path);
-				return NULL;
-			}
+			wstrlcat(path, bPtr->pathSeparator, slen);
 			if (i == column) {
 				item = lastItem;
 			} else {
@@ -786,10 +790,7 @@ WMArray *WMGetBrowserPaths(WMBrowser * bPtr)
 			}
 			if (!item)
 				break;
-			if (wstrlcat(path, item->text, slen) >= slen) {
-				wfree(path);
-				return NULL;
-			}
+			wstrlcat(path, item->text, slen);
 		}
 		WMAddToArray(paths, path);
 	}
