@@ -166,12 +166,15 @@ typedef struct W_Text {
 	WMArray *xdndDestinationTypes;
 } Text;
 
+/* not used */
+#if 0
 #define NOTIFY(T,C,N,A) {\
     WMNotification *notif = WMCreateNotification(N,T,A);\
     if ((T)->delegate && (T)->delegate->C)\
         (*(T)->delegate->C)((T)->delegate,notif);\
     WMPostNotification(notif);\
     WMReleaseNotification(notif);}
+#endif
 
 #define TYPETEXT 0
 
@@ -215,6 +218,11 @@ static char *default_bullet[] = {
 	"  ooo "
 };
 
+/* These id are used when sharing the selected text between applications */
+static Atom XA_Targets = None;
+static Atom XA_Format_Text = None;
+static Atom XA_Format_Compound_Text = None;
+
 static void handleEvents(XEvent * event, void *data);
 static void layOutDocument(Text * tPtr);
 static void updateScrollers(Text * tPtr);
@@ -617,7 +625,7 @@ static void paintText(Text * tPtr)
 	TextBlock *tb;
 	WMFont *font;
 	const char *text;
-	int len, y, c, s, done = False, dir /* 1 = down */ ;
+	int len, y, c, s, done = False, dir; /* dir 1 = down */
 	WMScreen *scr = tPtr->view->screen;
 	Display *dpy = tPtr->view->screen->display;
 	Window win = tPtr->view->window;
@@ -2036,17 +2044,13 @@ static void releaseSelection(Text * tPtr)
 static WMData *requestHandler(WMView * view, Atom selection, Atom target, void *cdata, Atom * type)
 {
 	Text *tPtr = view->self;
-	Display *dpy = tPtr->view->screen->display;
-	Atom _TARGETS;
-	Atom TEXT = XInternAtom(dpy, "TEXT", False);
-	Atom COMPOUND_TEXT = XInternAtom(dpy, "COMPOUND_TEXT", False);
 	WMData *data = NULL;
 
 	/* Parameter not used, but tell the compiler that it is ok */
 	(void) selection;
 	(void) cdata;
 
-	if (target == XA_STRING || target == TEXT || target == COMPOUND_TEXT) {
+	if (target == XA_STRING || target == XA_Format_Text || target == XA_Format_Compound_Text) {
 		char *text = WMGetTextSelectedStream(tPtr);
 
 		if (text) {
@@ -2059,14 +2063,13 @@ static WMData *requestHandler(WMView * view, Atom selection, Atom target, void *
 	} else
 		printf("didn't get it\n");
 
-	_TARGETS = XInternAtom(dpy, "TARGETS", False);
-	if (target == _TARGETS) {
+	if (target == XA_Targets) {
 		Atom array[4];
 
-		array[0] = _TARGETS;
+		array[0] = XA_Targets;
 		array[1] = XA_STRING;
-		array[2] = TEXT;
-		array[3] = COMPOUND_TEXT;
+		array[2] = XA_Format_Text;
+		array[3] = XA_Format_Compound_Text;
 
 		data = WMCreateDataWithBytes(&array, sizeof(array));
 		WMSetDataFormat(data, 32);
@@ -2305,6 +2308,7 @@ static void handleTextKeyPress(Text * tPtr, XEvent * event)
 
 	case XK_Return:
 		*buffer = '\n';
+		/* FALLTHRU */
 	default:
 		if (*buffer != 0 && !control_pressed) {
 			insertTextInteractively(tPtr, buffer, strlen(buffer));
@@ -2544,6 +2548,7 @@ static void handleActionEvents(XEvent * event, void *data)
 			break;
 		}
 
+	/* FALLTHRU */
 	case ButtonRelease:
 		if (tPtr->flags.pointerGrabbed) {
 			tPtr->flags.pointerGrabbed = False;
@@ -2972,6 +2977,17 @@ WMText *WMCreateTextForDocumentType(WMWidget * parent, WMAction * parser, WMActi
 	dpy = tPtr->view->screen->display;
 	scr = tPtr->view->screen;
 
+	if (XA_Targets == None) {
+		/*
+		 * Because the X protocol guaranties that the value will never change in
+		 * the lifespan of the server, we query the values only the first time a
+		 * widget is created
+		 */
+		XA_Targets = XInternAtom(dpy, "TARGETS", False);
+		XA_Format_Text = XInternAtom(dpy, "TEXT", False);
+		XA_Format_Compound_Text = XInternAtom(dpy, "COMPOUND_TEXT", False);
+	}
+
 	tPtr->view->self = tPtr;
 	tPtr->view->attribs.cursor = scr->textCursor;
 	tPtr->view->attribFlags |= CWOverrideRedirect | CWCursor;
