@@ -446,12 +446,18 @@ static void str2rcolor(RContext * rc, const char *name, RColor * color)
 {
 	XColor xcolor;
 
-	XParseColor(rc->dpy, rc->cmap, name, &xcolor);
-
-	color->alpha = 255;
-	color->red = xcolor.red >> 8;
-	color->green = xcolor.green >> 8;
-	color->blue = xcolor.blue >> 8;
+	if (XParseColor(rc->dpy, rc->cmap, name, &xcolor) != 0) {
+		color->alpha = 255;
+		color->red = xcolor.red >> 8;
+		color->green = xcolor.green >> 8;
+		color->blue = xcolor.blue >> 8;
+	} else {
+		/* Color Name was not found - Return white instead */
+		color->alpha = 255;
+		color->red   = 255;
+		color->green = 255;
+		color->blue  = 255;
+	}
 }
 
 static void dumpRImage(const char *path, RImage * image)
@@ -1070,6 +1076,8 @@ static void deleteTexture(WMWidget * w, void *data)
 	(void) w;
 
 	section = WMGetPopUpButtonSelectedItem(panel->secP);
+	if (section < 0)
+		return;
 	row = WMGetListSelectedItemRow(panel->texLs);
 	item = WMGetListItem(panel->texLs, row);
 	titem = (TextureListItem *) item->clientData;
@@ -1128,6 +1136,8 @@ static void changePage(WMWidget * w, void *data)
 
 	if (w) {
 		section = WMGetPopUpButtonSelectedItem(panel->secP);
+		if (section < 0)
+			return;
 
 		WMSelectListItem(panel->texLs, panel->textureIndex[section]);
 
@@ -1227,6 +1237,8 @@ static void textureDoubleClick(WMWidget * w, void *data)
 
 	/* unselect old texture */
 	section = WMGetPopUpButtonSelectedItem(panel->secP);
+	if (section < 0)
+		return;
 
 	item = WMGetListItem(panel->texLs, panel->textureIndex[section]);
 	titem = (TextureListItem *) item->clientData;
@@ -1255,7 +1267,7 @@ static void paintListItem(WMList * lPtr, int index, Drawable d, char *text, int
 {
 	_Panel *panel = (_Panel *) WMGetHangedData(lPtr);
 	WMScreen *scr = WMWidgetScreen(lPtr);
-	int width, height, x, y;
+	int width, height, x, y, tmp;
 	Display *dpy = WMScreenDisplay(scr);
 	WMColor *back = (state & WLDSSelected) ? WMWhiteColor(scr) : WMGrayColor(scr);
 	WMListItem *item;
@@ -1284,7 +1296,8 @@ static void paintListItem(WMList * lPtr, int index, Drawable d, char *text, int
 		XCopyArea(dpy, titem->preview, d, WMColorGC(black), 0, 0,
 			  TEXPREV_WIDTH, TEXPREV_HEIGHT, x + 5, y + 5);
 
-	if ((1 << WMGetPopUpButtonSelectedItem(panel->secP)) & titem->selectedFor)
+	tmp = WMGetPopUpButtonSelectedItem(panel->secP);
+	if ((tmp >= 0) && ((1 << tmp) & titem->selectedFor))
 		WMDrawPixmap(panel->onLed, d, x + TEXPREV_WIDTH + 10, y + 6);
 	else if (titem->selectedFor)
 		WMDrawPixmap(panel->offLed, d, x + TEXPREV_WIDTH + 10, y + 6);
@@ -1419,6 +1432,10 @@ static void changeColorPage(WMWidget * w, void *data)
 	WMScreen *scr = WMWidgetScreen(panel->box);
 	RContext *rc = WMScreenRContext(scr);
 
+	section = WMGetPopUpButtonSelectedItem(panel->colP);
+	if (section < 0)
+		return;
+
 	if (panel->preview) {
 		GC gc;
 
@@ -1430,7 +1447,6 @@ static void changeColorPage(WMWidget * w, void *data)
 			  colorOptions[panel->oldcsection].hand.y);
 	}
 	if (w) {
-		section = WMGetPopUpButtonSelectedItem(panel->colP);
 
 		panel->oldcsection = section;
 		if (panel->preview)
@@ -1731,6 +1747,8 @@ static void colorWellObserver(void *self, WMNotification * n)
 	(void) n;
 
 	p = WMGetPopUpButtonSelectedItem(panel->colP);
+	if (p < 0)
+		return;
 
 	WMReleaseColor(panel->colors[p]);
 
@@ -1822,10 +1840,10 @@ static void createPanel(Panel * p)
 	char *tmp;
 	Bool ok = True;
 
-	panel->fprefix = wstrconcat(wusergnusteppath(), "/Library/WindowMaker");
+	panel->fprefix = wstrconcat(wuserdatapath(), "/" PACKAGE_TARNAME);
 
 	if (access(panel->fprefix, F_OK) != 0) {
-		if (mkdir(panel->fprefix, 0755) < 0) {
+		if (-1 == mkdir(panel->fprefix, 0755) && errno != EEXIST) {
 			werror("%s", panel->fprefix);
 			ok = False;
 		}
@@ -1835,7 +1853,7 @@ static void createPanel(Panel * p)
 		wfree(panel->fprefix);
 		panel->fprefix = tmp;
 		if (access(panel->fprefix, F_OK) != 0) {
-			if (mkdir(panel->fprefix, 0755) < 0) {
+			if (-1 == mkdir(panel->fprefix, 0755) && errno != EEXIST) {
 				werror("%s", panel->fprefix);
 			}
 		}
