@@ -60,8 +60,6 @@ typedef struct _Panel {
 
 	CallbackRec callbacks;
 
-	Bool have_legacy_apercu;
-
 	WMWidget *parent;
 
 	WMFrame *posF;
@@ -75,12 +73,13 @@ typedef struct _Panel {
 	WMButton *posB[wlengthof_nocheck(icon_position_dbvalue)];
 
 	WMFrame *animF;
-	WMButton *animB[wlengthof_nocheck(icon_animation)];
+	WMPopUpButton *animP;
 
 	WMFrame *optF;
 	WMButton *arrB;
 	WMButton *omnB;
 	WMButton *sclB;
+	WMButton *marginB;
 
 	struct {
 		WMFrame *frame;
@@ -175,6 +174,7 @@ static void showData(_Panel * panel)
 	WMSetButtonSelected(panel->arrB, GetBoolForKey("AutoArrangeIcons"));
 	WMSetButtonSelected(panel->omnB, GetBoolForKey("StickyIcons"));
 	WMSetButtonSelected(panel->sclB, GetBoolForKey("SingleClickLaunch"));
+	WMSetButtonSelected(panel->marginB, GetBoolForKey("EnforceIconMargin"));
 
 	str = GetStringForKey("IconPosition");
 	if (str != NULL) {
@@ -195,20 +195,12 @@ static void showData(_Panel * panel)
 
 	if (i < 0)
 		i = 0;
-	else if (i > 9)
-		i = 9;
+	else if (i > 29)
+		i = 29;
 	WMSetPopUpButtonSelectedItem(panel->sizeP, i);
 
 	/* Mini-Previews for Icons */
 
-	/*
-	 * Backward Compatibility:
-	 * These settings changed names after 0.95.6, so to avoid breaking user's
-	 * config we still support the old names, and silently convert them to the
-	 * new settings
-	 * This hack should be kept for at least 2 years, that means >= 2017.
-	 */
-	panel->have_legacy_apercu = False;
 	str = GetStringForKey("MiniwindowPreviewBalloons");
 	if (str != NULL) {
 		/* New names found, use them in priority */
@@ -220,25 +212,6 @@ static void showData(_Panel * panel)
 		} else {
 			i = minipreview_minimum_size;
 		}
-	} else {
-		/* No new names, try the legacy names */
-		b = GetBoolForKey("MiniwindowApercuBalloons");
-		if (b) {
-			panel->have_legacy_apercu = True;
-			i = GetIntegerForKey("ApercuSize");
-
-			/*
-			 * In the beginning, the option was coded as a multiple of the icon
-			 * size; then it was converted to pixel size
-			 */
-			if (i < 24)
-				i *= GetIntegerForKey("IconSize");
-
-			if (i <= minipreview_minimum_size)
-				i = minipreview_minimum_size + 1;	/* +1 to not display as "off" */
-		} else {
-			i = minipreview_minimum_size;
-		}
 	}
 	WMSetSliderValue(panel->minipreview.slider, i);
 	minipreview_slider_changed(panel->minipreview.slider, panel);
@@ -248,7 +221,7 @@ static void showData(_Panel * panel)
 	if (str != NULL) {
 		for (i = 0; i < wlengthof(icon_animation); i++) {
 			if (strcasecmp(str, icon_animation[i].db_value) == 0) {
-				WMPerformButtonClick(panel->animB[i]);
+				WMSetPopUpButtonSelectedItem(panel->animP, i);
 				goto found_animation_value;
 			}
 		}
@@ -256,7 +229,7 @@ static void showData(_Panel * panel)
 		         str, icon_animation[0].db_value);
 	}
 	/* If we're here, no valid value have been found so we fall-back to the default */
-	WMPerformButtonClick(panel->animB[0]);
+	WMSetPopUpButtonSelectedItem(panel->animP, 0);
  found_animation_value:
 	;
 }
@@ -366,7 +339,7 @@ static void createPanel(Panel * p)
 	panel->sizeP = WMCreatePopUpButton(panel->sizeF);
 	WMResizeWidget(panel->sizeP, 80, 20);
 	WMMoveWidget(panel->sizeP, 10, 19);
-	for (i = 24; i <= 96; i += 8) {
+	for (i = 24; i <= 256; i += 8) {
 		sprintf(buf, "%ix%i", i, i);
 		WMAddPopUpButtonItem(panel->sizeP, buf);
 	}
@@ -399,50 +372,57 @@ static void createPanel(Panel * p)
 
     /***************** Animation ****************/
 	panel->animF = WMCreateFrame(panel->box);
-	WMResizeWidget(panel->animF, 215, 110);
+	WMResizeWidget(panel->animF, 215, 52);
 	WMMoveWidget(panel->animF, 292, 6);
 	WMSetFrameTitle(panel->animF, _("Iconification Animation"));
 
-	for (i = 0; i < wlengthof(icon_animation); i++) {
-		panel->animB[i] = WMCreateRadioButton(panel->animF);
-		WMResizeWidget(panel->animB[i], 192, 20);
-		WMMoveWidget(panel->animB[i], 12, 16 + i * 22);
-
-		if (i > 0)
-			WMGroupButtons(panel->animB[0], panel->animB[i]);
-
-		WMSetButtonText(panel->animB[i], _(icon_animation[i].label));
-	}
+	panel->animP = WMCreatePopUpButton(panel->animF);
+	WMResizeWidget(panel->animP, 195, 20);
+	WMMoveWidget(panel->animP, 10, 19);
+	for (i = 0; i < wlengthof(icon_animation); i++)
+		WMAddPopUpButtonItem(panel->animP, _(icon_animation[i].label));
 
 	WMMapSubwidgets(panel->animF);
 
     /***************** Options ****************/
 	panel->optF = WMCreateFrame(panel->box);
-	WMResizeWidget(panel->optF, 215, 90);
-	WMMoveWidget(panel->optF, 292, 130);
+	WMResizeWidget(panel->optF, 215, 148);
+	WMMoveWidget(panel->optF, 292, 72);
 	/*    WMSetFrameTitle(panel->optF, _("Icon Display")); */
+	starty = 8 + 14;   /* the last term centers the checkboxes within the panel; subtract 13 for a new option */
 
 	panel->arrB = WMCreateSwitchButton(panel->optF);
 	WMResizeWidget(panel->arrB, 198, 26);
-	WMMoveWidget(panel->arrB, 12, 8);
+	WMMoveWidget(panel->arrB, 12, starty);
+	starty += 26;
 	WMSetButtonText(panel->arrB, _("Auto-arrange icons"));
 
 	WMSetBalloonTextForView(_("Keep icons and miniwindows arranged all the time."), WMWidgetView(panel->arrB));
 
 	panel->omnB = WMCreateSwitchButton(panel->optF);
 	WMResizeWidget(panel->omnB, 198, 26);
-	WMMoveWidget(panel->omnB, 12, 34);
+	WMMoveWidget(panel->omnB, 12, starty);
+	starty += 26;
 	WMSetButtonText(panel->omnB, _("Omnipresent miniwindows"));
 
 	WMSetBalloonTextForView(_("Make miniwindows be present in all workspaces."), WMWidgetView(panel->omnB));
 
 	panel->sclB = WMCreateSwitchButton(panel->optF);
 	WMResizeWidget(panel->sclB, 198, 26);
-	WMMoveWidget(panel->sclB, 12, 60);
+	WMMoveWidget(panel->sclB, 12, starty);
+	starty += 26;
 	WMSetButtonText(panel->sclB, _("Single click activation"));
 
 	WMSetBalloonTextForView(_("Launch applications and restore windows with a single click."), WMWidgetView(panel->sclB));
 
+	panel->marginB = WMCreateSwitchButton(panel->optF);
+	WMResizeWidget(panel->marginB, 198, 26);
+	WMMoveWidget(panel->marginB, 12, starty);
+	starty += 26;
+	WMSetButtonText(panel->marginB, _("Enforce icon margin"));
+
+	WMSetBalloonTextForView(_("Make sure that the icon image does not protrude into the icon frame."), WMWidgetView(panel->marginB));
+
 	WMMapSubwidgets(panel->optF);
 
 	WMRealizeWidget(panel->box);
@@ -458,6 +438,7 @@ static void storeData(_Panel * panel)
 	SetBoolForKey(WMGetButtonSelected(panel->arrB), "AutoArrangeIcons");
 	SetBoolForKey(WMGetButtonSelected(panel->omnB), "StickyIcons");
 	SetBoolForKey(WMGetButtonSelected(panel->sclB), "SingleClickLaunch");
+	SetBoolForKey(WMGetButtonSelected(panel->marginB), "EnforceIconMargin");
 
 	SetIntegerForKey(WMGetPopUpButtonSelectedItem(panel->sizeP) * 8 + 24, "IconSize");
 
@@ -478,17 +459,8 @@ static void storeData(_Panel * panel)
 		}
 		SetIntegerForKey(i, "MiniPreviewSize");
 	}
-	if (panel->have_legacy_apercu) {
-		RemoveObjectForKey("MiniwindowApercuBalloons");
-		RemoveObjectForKey("ApercuSize");
-	}
 
-	for (i = 0; i < wlengthof(icon_animation); i++) {
-		if (WMGetButtonSelected(panel->animB[i])) {
-			SetStringForKey(icon_animation[i].db_value, "IconificationStyle");
-			break;
-		}
-	}
+	SetStringForKey(icon_animation[WMGetPopUpButtonSelectedItem(panel->animP)].db_value, "IconificationStyle");
 }
 
 Panel *InitIcons(WMWidget *parent)
