@@ -291,6 +291,8 @@ static void changedItemPad(WMWidget * w, void *data)
 	_Panel *panel = (_Panel *) data;
 	int padn = WMGetPopUpButtonSelectedItem(w);
 
+	if (padn < 0)
+		return;
 	WMUnmapWidget(panel->itemPad[panel->currentPad]);
 	WMMapWidget(panel->itemPad[padn]);
 
@@ -507,7 +509,7 @@ static void createPanel(_Panel * p)
 		WMSetScrollViewContentView(sview, WMWidgetView(pad));
 
 		data = putNewItem(panel, pad, ExternalInfo, _("Debian Menu"));
-		data->param.pipe.command = "/etc/GNUstep/Defaults/menu.hook";
+		data->param.pipe.command = "/etc/" GSUSER_SUBDIR "/" DEFAULTS_SUBDIR "/menu.hook";
 
 		data = putNewItem(panel, pad, PipeInfo, _("RedHat Menu"));
 		data->param.pipe.command = "wmconfig --output wmaker";
@@ -518,19 +520,25 @@ static void createPanel(_Panel * p)
 		data = putNewItem(panel, pad, DirectoryInfo, _("Themes"));
 		data->param.directory.command = "setstyle";
 		data->param.directory.directory =
-		    "/usr/share/WindowMaker/Themes /usr/local/share/WindowMaker/Themes $HOME/GNUstep/Library/WindowMaker/Themes";
+		    "/usr/share/" PACKAGE_TARNAME "/Themes"
+		    " /usr/local/share/" PACKAGE_TARNAME "/Themes"
+		    " $HOME/" GSUSER_SUBDIR "/" USERDATA_SUBDIR "/" PACKAGE_TARNAME "/Themes";
 		data->param.directory.stripExt = 1;
 
 		data = putNewItem(panel, pad, DirectoryInfo, _("Bg Images (scale)"));
 		data->param.directory.command = "wmsetbg -u -s";
 		data->param.directory.directory =
-		    "/opt/kde2/share/wallpapers /usr/share/WindowMaker/Backgrounds $HOME/GNUstep/Library/WindowMaker/Backgrounds";
+		    "/opt/kde2/share/wallpapers"
+		    " /usr/share/" PACKAGE_TARNAME "/Backgrounds"
+		    " $HOME/" GSUSER_SUBDIR "/" USERDATA_SUBDIR "/" PACKAGE_TARNAME "/Backgrounds";
 		data->param.directory.stripExt = 1;
 
 		data = putNewItem(panel, pad, DirectoryInfo, _("Bg Images (tile)"));
 		data->param.directory.command = "wmsetbg -u -t";
 		data->param.directory.directory =
-		    "/opt/kde2/share/wallpapers /usr/share/WindowMaker/Backgrounds $HOME/GNUstep/Library/WindowMaker/Backgrounds";
+		    "/opt/kde2/share/wallpapers"
+		    " /usr/share/" PACKAGE_TARNAME "/Backgrounds"
+		    " $HOME/" GSUSER_SUBDIR "/" USERDATA_SUBDIR "/" PACKAGE_TARNAME "/Backgrounds";
 		data->param.directory.stripExt = 1;
 
 		smenu = putNewSubmenu(pad, _("Assorted XTerms"));
@@ -619,7 +627,8 @@ static void createPanel(_Panel * p)
 	WMSetLabelText(label, _("Enter the path for a file containing a menu\n"
 				"or a list of directories with the programs you\n"
 				"want to have listed in the menu. Ex:\n"
-				"~/GNUstep/Library/WindowMaker/menu\n" "or\n" "/usr/bin ~/xbin"));
+				"~/" GSUSER_SUBDIR "/" USERDATA_SUBDIR "/" PACKAGE_TARNAME "/menu\n"
+				"or\n" "/usr/bin ~/xbin"));
 
 	WMMapSubwidgets(panel->pathF);
 
@@ -914,6 +923,11 @@ static ItemData *parseCommand(WMPropList * item)
 
 		data->type = ExecInfo;
 
+		if (parameter == NULL) {
+			wfree(data);
+			return NULL;
+		}
+
 		data->param.exec.command = wstrdup(parameter);
 		if (shortcut)
 			data->param.exec.shortcut = wstrdup(shortcut);
@@ -926,6 +940,12 @@ static ItemData *parseCommand(WMPropList * item)
 		 * |pipe
 		 */
 		p = parameter;
+
+		if (p == NULL) {
+			wfree(data);
+			return NULL;
+		}
+
 		while (isspace(*p) && *p)
 			p++;
 		if (*p == '|') {
@@ -1482,16 +1502,11 @@ static WMPropList *getDefaultMenu(_Panel * panel)
 
 static void showData(_Panel * panel)
 {
-	const char *gspath;
 	char *menuPath, *labelText;
 	char buf[1024];
 	WMPropList *pmenu;
 
-	gspath = wusergnusteppath();
-
-	menuPath = wmalloc(strlen(gspath) + 32);
-	strcpy(menuPath, gspath);
-	strcat(menuPath, "/Defaults/WMRootMenu");
+	menuPath = wdefaultspathfordomain("WMRootMenu");
 
 	pmenu = WMReadPropListFromFile(menuPath);
 
@@ -1502,8 +1517,12 @@ static void showData(_Panel * panel)
 
 		path = wexpandpath(WMGetFromPLString(pmenu));
 
-		if (access(path, F_OK) < 0)
+		if (access(path, F_OK) < 0) {
+			char *old_path = path;
+
 			path = wfindfile(DEF_CONFIG_PATHS, path);
+			wfree(old_path);
+		}
 
 		/* TODO: if needed, concatenate locale suffix to path.
 		   See getLocalizedMenuFile() in src/rootmenu.c. */
