@@ -26,11 +26,52 @@
 #include <X11/XKBlib.h>
 
 #include <unistd.h>
+#include <sys/types.h>
+#include <sys/stat.h>
 #include <math.h>
 
 
 #define XSET	"xset"
 
+
+static const struct {
+	const char *db_key;
+	int default_action;
+	enum { T_BUTTON, T_WHEEL } type;
+	const char *display_label;
+} button_list[] = {
+	{ "MouseLeftButtonAction",     3, T_BUTTON, N_("Left Button") },
+	{ "MouseMiddleButtonAction",   2, T_BUTTON, N_("Middle Button") },
+	{ "MouseRightButtonAction",    1, T_BUTTON, N_("Right Button") },
+	{ "MouseBackwardButtonAction", 0, T_BUTTON, N_("Back Button") },
+	{ "MouseForwardButtonAction",  0, T_BUTTON, N_("Forward Button") },
+	{ "MouseWheelAction",          0, T_WHEEL,  N_("Mouse Wheel") },
+	{ "MouseWheelTiltAction",      0, T_WHEEL,  N_("Mouse Wheel Tilt") }
+};
+
+static const struct {
+	const char *db_value;
+	const char *label;
+} button_actions[] = {
+	{ "None",                  N_("None") },
+	{ "OpenApplicationsMenu",  N_("Applications Menu") },
+	{ "OpenWindowListMenu",    N_("Window List Menu") },
+	{ "SelectWindows",         N_("Select Windows") },
+	{ "MoveToPrevWorkspace",   N_("Previous Workspace") },
+	{ "MoveToNextWorkspace",   N_("Next Workspace") },
+	{ "MoveToPrevWindow",      N_("Previous Window") },
+	{ "MoveToNextWindow",      N_("Next Window") }
+};
+
+static const struct {
+	const char *db_value;
+	const char *label;
+} wheel_actions[] = {
+	{ "None",              N_("None") },
+	{ "SwitchWorkspaces",  N_("Switch Workspaces") },
+	{ "SwitchWindows",     N_("Switch Windows") }
+};
+
 typedef struct _Panel {
 	WMBox *box;
 
@@ -57,14 +98,10 @@ typedef struct _Panel {
 	DoubleTest *tester;
 
 	WMFrame *menuF;
-	WMLabel *button1L;
-	WMLabel *button2L;
-	WMLabel *button3L;
-	WMLabel *wheelL;
-	WMPopUpButton *button1P;
-	WMPopUpButton *button2P;
-	WMPopUpButton *button3P;
-	WMPopUpButton *wheelP;
+	struct {
+		WMLabel *label;
+		WMPopUpButton *popup;
+	} mouse_action[wlengthof_nocheck(button_list)];
 
 	WMButton *disaB;
 
@@ -87,10 +124,6 @@ typedef struct _Panel {
 
 static char *modifierNames[8];
 
-static char *buttonActions[4];
-
-static char *wheelActions[2];
-
 #define DELAY(i)		((i)*75+170)
 
 
@@ -117,7 +150,7 @@ static void speedChange(WMWidget * w, void *data)
 
 		tmp = WMGetTextFieldText(panel->acceT);
 		if (sscanf(tmp, "%f", &accel) != 1 || accel < 0) {
-			WMRunAlertPanel(WMWidgetScreen(panel->acceT), GetWindow(panel),
+			WMRunAlertPanel(WMWidgetScreen(panel->acceT), GetWindow(),
 					_("Error"),
 					_("Invalid mouse acceleration value. Must be a positive real value."),
 					_("OK"), NULL, NULL);
@@ -137,7 +170,7 @@ static void speedChange(WMWidget * w, void *data)
 
 	tmp = WMGetTextFieldText(panel->threT);
 	if (sscanf(tmp, "%i", &threshold) != 1 || threshold < 0 || threshold > panel->maxThreshold) {
-		WMRunAlertPanel(WMWidgetScreen(panel->parent), GetWindow(panel), _("Error"),
+		WMRunAlertPanel(WMWidgetScreen(panel->parent), GetWindow(), _("Error"),
 				_
 				("Invalid mouse acceleration threshold value. Must be the number of pixels to travel before accelerating."),
 				_("OK"), NULL, NULL);
@@ -151,16 +184,35 @@ static void returnPressed(void *observerData, WMNotification * notification)
 {
 	_Panel *panel = (_Panel *) observerData;
 
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) notification;
+
 	speedChange(NULL, panel);
 }
 
+static void toggle_disabling_of_mouse_actions(WMWidget *w, void *client_data)
+{
+	WMButton *button = (WMButton *) w;
+	Panel *panel = (Panel *) client_data;
+	Bool do_enable;
+	int i;
+
+	if (WMGetButtonSelected(button))
+		do_enable = False;
+	else
+		do_enable = True;
+
+	for (i = 0; i < wlengthof(panel->mouse_action); i++)
+		WMSetPopUpButtonEnabled(panel->mouse_action[i].popup, do_enable);
+}
+
 static void doubleClick(WMWidget * w, void *data)
 {
 	_Panel *panel = (_Panel *) data;
 	int i;
 	char buffer[32];
 
-	for (i = 0; i < 5; i++) {
+	for (i = 0; i < wlengthof(panel->ddelaB); i++) {
 		if (panel->ddelaB[i] == w)
 			break;
 	}
@@ -172,34 +224,32 @@ static void doubleClick(WMWidget * w, void *data)
 
 static int getButtonAction(const char *str)
 {
+	int i;
+
 	if (!str)
 		return -2;
 
-	if (strcasecmp(str, "None") == 0)
-		return 0;
-	else if (strcasecmp(str, "OpenApplicationsMenu") == 0)
-		return 1;
-	else if (strcasecmp(str, "OpenWindowListMenu") == 0)
-		return 2;
-	else if (strcasecmp(str, "SelectWindows") == 0)
-		return 3;
-	else
-		return -1;
+	for (i = 0; i < wlengthof(button_actions); i++) {
+		if (strcasecmp(str, button_actions[i].db_value) == 0)
+			return i;
+	}
 
+	return -1;
 }
 
 static int getWheelAction(const char *str)
 {
+	int i;
+
 	if (!str)
 		return -2;
 
-	if (strcasecmp(str, "None") == 0)
-		return 0;
-	else if (strcasecmp(str, "SwitchWorkspaces") == 0)
-		return 1;
-	else
-		return -1;
+	for (i = 0; i < wlengthof(wheel_actions); i++) {
+		if (strcasecmp(str, wheel_actions[i].db_value) == 0)
+			return i;
+	}
 
+	return -1;
 }
 
 static void getMouseParameters(Display * dpy, float *accel, int *thre)
@@ -215,60 +265,30 @@ static void showData(_Panel * panel)
 {
 	char *str;
 	int i;
-	int a = -1, b = -1, c = -1, w = -1;
+	int a = -1, b = -1;
 	float accel;
 	char buffer[32];
 	Display *dpy = WMScreenDisplay(WMWidgetScreen(panel->parent));
 
-	str = GetStringForKey("MouseLeftButtonAction");
-	i = getButtonAction(str);
-	if (i < 0) {
-		a = 3;
-		if (i == -1) {
-			wwarning(_("bad value %s for option %s"), str, "MouseLeftButtonAction");
-		}
-	} else {
-		a = i;
-	}
-	WMSetPopUpButtonSelectedItem(panel->button1P, a);
+	for (i = 0; i < wlengthof(button_list); i++) {
+		int action;
 
-	str = GetStringForKey("MouseMiddleButtonAction");
-	i = getButtonAction(str);
-	if (i < 0) {
-		b = 2;
-		if (i == -1) {
-			wwarning(_("bad value %s for option %s"), str, "MouseMiddleButtonAction");
-		}
-	} else {
-		b = i;
-	}
-	WMSetPopUpButtonSelectedItem(panel->button2P, b);
+		str = GetStringForKey(button_list[i].db_key);
+		if (button_list[i].type == T_BUTTON)
+			action = getButtonAction(str);
+		else
+			action = getWheelAction(str);
 
-	str = GetStringForKey("MouseRightButtonAction");
-	i = getButtonAction(str);
-	if (i < 0) {
-		c = 1;
-		if (i == -1) {
-			wwarning(_("bad value %s for option %s"), str, "MouseRightButtonAction");
+		if (action < 0) {
+			if (action == -1)
+				wwarning(_("bad value %s for option %s"), str, button_list[i].db_key);
+			action = button_list[i].default_action;
 		}
-	} else {
-		c = i;
+		WMSetPopUpButtonSelectedItem(panel->mouse_action[i].popup, action);
 	}
-	WMSetPopUpButtonSelectedItem(panel->button3P, c);
-
-	str = GetStringForKey("MouseWheelAction");
-	i = getWheelAction(str);
-	if (i < 0) {
-		w = 0;
-		if (i == -1) {
-			wwarning(_("bad value %s for option %s"), str, "MouseWheelAction");
-		}
-	} else {
-		w = i;
-	}
-	WMSetPopUpButtonSelectedItem(panel->wheelP, w);
 
 	WMSetButtonSelected(panel->disaB, GetBoolForKey("DisableWSMouseActions"));
+	toggle_disabling_of_mouse_actions(panel->disaB, panel);
 
 	 /**/ getMouseParameters(dpy, &accel, &a);
 	panel->maxThreshold = WidthOfScreen(DefaultScreenOfDisplay(dpy));
@@ -278,16 +298,16 @@ static void showData(_Panel * panel)
 	sprintf(buffer, "%i", a);
 	WMSetTextFieldText(panel->threT, buffer);
 
-	WMSetSliderValue(panel->speedS, (accel - 0.25) / 0.25);
+	WMSetSliderValue(panel->speedS, (accel - 0.25F) / 0.25F);
 
 	panel->acceleration = accel;
-	sprintf(buffer, "%.2f", accel);
+	sprintf(buffer, "%.2f", (double)accel);
 	WMSetTextFieldText(panel->acceT, buffer);
 
 	 /**/ b = GetIntegerForKey("DoubleClickTime");
 	/* find best match */
 	a = -1;
-	for (i = 0; i < 5; i++) {
+	for (i = 0; i < wlengthof(panel->ddelaB); i++) {
 		if (DELAY(i) == b)
 			a = i;
 	}
@@ -361,10 +381,7 @@ static void fillModifierPopUp(WMPopUpButton * pop)
 			if (mapping->modifiermap[idx] != 0) {
 				int l;
 				for (l = 0; l < 4; l++) {
-					if (xext_xkb_supported)
-						ksym = XkbKeycodeToKeysym(dpy, mapping->modifiermap[idx], 0, l);
-					else
-						ksym = XKeycodeToKeysym(dpy, mapping->modifiermap[idx], 0);
+					ksym = W_KeycodeToKeysym(dpy, mapping->modifiermap[idx], l);
 					if (ksym != NoSymbol)
 						break;
 				}
@@ -433,13 +450,13 @@ static void createPanel(Panel * p)
 
     /**************** Mouse Speed ****************/
 	panel->speedF = WMCreateFrame(panel->box);
-	WMResizeWidget(panel->speedF, 245, 100);
-	WMMoveWidget(panel->speedF, 15, 5);
+	WMResizeWidget(panel->speedF, 219, 85);
+	WMMoveWidget(panel->speedF, 9, 54);
 	WMSetFrameTitle(panel->speedF, _("Mouse Speed"));
 
 	panel->speedL = WMCreateLabel(panel->speedF);
 	WMResizeWidget(panel->speedL, 40, 46);
-	WMMoveWidget(panel->speedL, 15, 14);
+	WMMoveWidget(panel->speedL, 8, 10);
 	WMSetLabelImagePosition(panel->speedL, WIPImageOnly);
 	path = LocateImage(SPEED_ICON_FILE);
 	if (path) {
@@ -454,51 +471,68 @@ static void createPanel(Panel * p)
 	}
 
 	panel->speedS = WMCreateSlider(panel->speedF);
-	WMResizeWidget(panel->speedS, 160, 15);
-	WMMoveWidget(panel->speedS, 70, 35);
+	WMResizeWidget(panel->speedS, 150, 15);
+	WMMoveWidget(panel->speedS, 58, 30);
 	WMSetSliderMinValue(panel->speedS, 0);
 	WMSetSliderMaxValue(panel->speedS, 40);
 	WMSetSliderContinuous(panel->speedS, False);
 	WMSetSliderAction(panel->speedS, speedChange, panel);
 
 	panel->acceL = WMCreateLabel(panel->speedF);
-	WMResizeWidget(panel->acceL, 70, 16);
-	WMMoveWidget(panel->acceL, 10, 67);
+	WMResizeWidget(panel->acceL, 50, 16);
+	WMMoveWidget(panel->acceL, 8, 58);
 	WMSetLabelTextAlignment(panel->acceL, WARight);
-	WMSetLabelText(panel->acceL, _("Acceler.:"));
+	WMSetLabelText(panel->acceL, _("Accel.:"));
 
 	panel->acceT = WMCreateTextField(panel->speedF);
 	WMResizeWidget(panel->acceT, 40, 20);
-	WMMoveWidget(panel->acceT, 80, 65);
+	WMMoveWidget(panel->acceT, 58, 56);
 	WMAddNotificationObserver(returnPressed, panel, WMTextDidEndEditingNotification, panel->acceT);
 
 	panel->threL = WMCreateLabel(panel->speedF);
 	WMResizeWidget(panel->threL, 80, 16);
-	WMMoveWidget(panel->threL, 120, 67);
+	WMMoveWidget(panel->threL, 98, 58);
 	WMSetLabelTextAlignment(panel->threL, WARight);
 	WMSetLabelText(panel->threL, _("Threshold:"));
 
 	panel->threT = WMCreateTextField(panel->speedF);
 	WMResizeWidget(panel->threT, 30, 20);
-	WMMoveWidget(panel->threT, 200, 65);
+	WMMoveWidget(panel->threT, 178, 56);
 	WMAddNotificationObserver(returnPressed, panel, WMTextDidEndEditingNotification, panel->threT);
 
 	WMMapSubwidgets(panel->speedF);
 
-    /***************** Doubleclick Delay ****************/
+	/* ************** Grab Modifier **************** */
+	panel->grabF = WMCreateFrame(panel->box);
+	WMResizeWidget(panel->grabF, 219, 46);
+	WMMoveWidget(panel->grabF, 9, 5);
+	WMSetFrameTitle(panel->grabF, _("Mouse Grab Modifier"));
+
+	WMSetBalloonTextForView(_("Keyboard modifier to use for actions that\n"
+				  "involve dragging windows with the mouse,\n"
+				  "clicking inside the window."), WMWidgetView(panel->grabF));
+
+	panel->grabP = WMCreatePopUpButton(panel->grabF);
+	WMResizeWidget(panel->grabP, 178, 20);
+	WMMoveWidget(panel->grabP, 20, 17);
+
+	fillModifierPopUp(panel->grabP);
+	WMMapSubwidgets(panel->grabF);
+
+	/***************** Doubleclick Delay ****************/
 
 	panel->ddelaF = WMCreateFrame(panel->box);
-	WMResizeWidget(panel->ddelaF, 245, 105);
-	WMMoveWidget(panel->ddelaF, 15, 115);
+	WMResizeWidget(panel->ddelaF, 219, 80);
+	WMMoveWidget(panel->ddelaF, 9, 142);
 	WMSetFrameTitle(panel->ddelaF, _("Double-Click Delay"));
 
 	buf1 = wmalloc(strlen(DELAY_ICON) + 2);
 	buf2 = wmalloc(strlen(DELAY_ICON_S) + 2);
 
-	for (i = 0; i < 5; i++) {
+	for (i = 0; i < wlengthof(panel->ddelaB); i++) {
 		panel->ddelaB[i] = WMCreateCustomButton(panel->ddelaF, WBBStateChangeMask);
 		WMResizeWidget(panel->ddelaB[i], 25, 25);
-		WMMoveWidget(panel->ddelaB[i], 30 + (40 * i), 25);
+		WMMoveWidget(panel->ddelaB[i], 18 + (40 * i), 18);
 		WMSetButtonBordered(panel->ddelaB[i], False);
 		WMSetButtonImagePosition(panel->ddelaB[i], WIPImageOnly);
 		WMSetButtonAction(panel->ddelaB[i], doubleClick, panel);
@@ -534,16 +568,16 @@ static void createPanel(Panel * p)
 	wfree(buf2);
 
 	panel->tester = CreateDoubleTest(panel->ddelaF, _("Test"));
-	WMResizeWidget(panel->tester, 84, 29);
-	WMMoveWidget(panel->tester, 35, 60);
+	WMResizeWidget(panel->tester, 84, 24);
+	WMMoveWidget(panel->tester, 20, 48);
 
 	panel->ddelaT = WMCreateTextField(panel->ddelaF);
 	WMResizeWidget(panel->ddelaT, 40, 20);
-	WMMoveWidget(panel->ddelaT, 140, 65);
+	WMMoveWidget(panel->ddelaT, 130, 50);
 
 	panel->ddelaL = WMCreateLabel(panel->ddelaF);
 	WMResizeWidget(panel->ddelaL, 40, 16);
-	WMMoveWidget(panel->ddelaL, 185, 70);
+	WMMoveWidget(panel->ddelaL, 173, 54);
 	{
 		WMFont *font;
 		WMColor *color;
@@ -555,91 +589,46 @@ static void createPanel(Panel * p)
 		WMReleaseFont(font);
 		WMReleaseColor(color);
 	}
-	WMSetLabelText(panel->ddelaL, _("msec"));
+	WMSetLabelText(panel->ddelaL, _("ms"));
 
 	WMMapSubwidgets(panel->ddelaF);
 
 	/* ************** Workspace Action Buttons **************** */
+
 	panel->menuF = WMCreateFrame(panel->box);
-	WMResizeWidget(panel->menuF, 240, 160);
-	WMMoveWidget(panel->menuF, 270, 5);
+	WMResizeWidget(panel->menuF, 276, 217);
+	WMMoveWidget(panel->menuF, 236, 5);
 	WMSetFrameTitle(panel->menuF, _("Workspace Mouse Actions"));
 
 	panel->disaB = WMCreateSwitchButton(panel->menuF);
-	WMResizeWidget(panel->disaB, 205, 18);
-	WMMoveWidget(panel->disaB, 10, 18);
+	WMResizeWidget(panel->disaB, 254, 18);
+	WMMoveWidget(panel->disaB, 10, 15);
 	WMSetButtonText(panel->disaB, _("Disable mouse actions"));
+	WMSetButtonAction(panel->disaB, toggle_disabling_of_mouse_actions, panel);
 
-	panel->button1L = WMCreateLabel(panel->menuF);
-	WMResizeWidget(panel->button1L, 87, 20);
-	WMMoveWidget(panel->button1L, 5, 45);
-	WMSetLabelTextAlignment(panel->button1L, WARight);
-	WMSetLabelText(panel->button1L, _("Left Button"));
+	for (i = 0; i < wlengthof(button_list); i++) {
+		int j;
 
-	panel->button1P = WMCreatePopUpButton(panel->menuF);
-	WMResizeWidget(panel->button1P, 135, 20);
-	WMMoveWidget(panel->button1P, 95, 45);
+		panel->mouse_action[i].label = WMCreateLabel(panel->menuF);
+		WMResizeWidget(panel->mouse_action[i].label, 115, 20);
+		WMMoveWidget(panel->mouse_action[i].label, 4, 37 + 25 * i);
+		WMSetLabelTextAlignment(panel->mouse_action[i].label, WARight);
+		WMSetLabelText(panel->mouse_action[i].label, _(button_list[i].display_label));
 
-	panel->button2L = WMCreateLabel(panel->menuF);
-	WMResizeWidget(panel->button2L, 87, 20);
-	WMMoveWidget(panel->button2L, 5, 73);
-	WMSetLabelTextAlignment(panel->button2L, WARight);
-	WMSetLabelText(panel->button2L, _("Middle Button"));
+		panel->mouse_action[i].popup = WMCreatePopUpButton(panel->menuF);
+		WMResizeWidget(panel->mouse_action[i].popup, 145, 20);
+		WMMoveWidget(panel->mouse_action[i].popup, 121, 37 + 25 * i);
 
-	panel->button2P = WMCreatePopUpButton(panel->menuF);
-	WMResizeWidget(panel->button2P, 135, 20);
-	WMMoveWidget(panel->button2P, 95, 73);
-
-	panel->button3L = WMCreateLabel(panel->menuF);
-	WMResizeWidget(panel->button3L, 87, 20);
-	WMMoveWidget(panel->button3L, 5, 101);
-	WMSetLabelTextAlignment(panel->button3L, WARight);
-	WMSetLabelText(panel->button3L, _("Right Button"));
-
-	panel->button3P = WMCreatePopUpButton(panel->menuF);
-	WMResizeWidget(panel->button3P, 135, 20);
-	WMMoveWidget(panel->button3P, 95, 101);
-
-	panel->wheelL = WMCreateLabel(panel->menuF);
-	WMResizeWidget(panel->wheelL, 87, 20);
-	WMMoveWidget(panel->wheelL, 5, 129);
-	WMSetLabelTextAlignment(panel->wheelL, WARight);
-	WMSetLabelText(panel->wheelL, _("Mouse Wheel"));
-
-	panel->wheelP = WMCreatePopUpButton(panel->menuF);
-	WMResizeWidget(panel->wheelP, 135, 20);
-	WMMoveWidget(panel->wheelP, 95, 129);
-
-	for (i = 0; i < sizeof(buttonActions) / sizeof(buttonActions[0]); i++) {
-		WMAddPopUpButtonItem(panel->button1P, buttonActions[i]);
-		WMAddPopUpButtonItem(panel->button2P, buttonActions[i]);
-		WMAddPopUpButtonItem(panel->button3P, buttonActions[i]);
+		if (button_list[i].type == T_BUTTON) {
+			for (j = 0; j < wlengthof(button_actions); j++)
+				WMAddPopUpButtonItem(panel->mouse_action[i].popup, _(button_actions[j].label));
+		} else {
+			for (j = 0; j < wlengthof(wheel_actions); j++)
+				WMAddPopUpButtonItem(panel->mouse_action[i].popup, _(wheel_actions[j].label));
+		}
 	}
-
-	for (i = 0; i < sizeof(wheelActions) / sizeof(wheelActions[0]); i++) {
-		WMAddPopUpButtonItem(panel->wheelP, wheelActions[i]);
-	}
-
 	WMMapSubwidgets(panel->menuF);
 
-	/* ************** Grab Modifier **************** */
-	panel->grabF = WMCreateFrame(panel->box);
-	WMResizeWidget(panel->grabF, 240, 50);
-	WMMoveWidget(panel->grabF, 270, 170);
-	WMSetFrameTitle(panel->grabF, _("Mouse Grab Modifier"));
-
-	WMSetBalloonTextForView(_("Keyboard modifier to use for actions that\n"
-				  "involve dragging windows with the mouse,\n"
-				  "clicking inside the window."), WMWidgetView(panel->grabF));
-
-	panel->grabP = WMCreatePopUpButton(panel->grabF);
-	WMResizeWidget(panel->grabP, 160, 20);
-	WMMoveWidget(panel->grabP, 40, 20);
-
-	fillModifierPopUp(panel->grabP);
-
-	WMMapSubwidgets(panel->grabF);
-
 	WMRealizeWidget(panel->box);
 	WMMapSubwidgets(panel->box);
 
@@ -651,8 +640,14 @@ static void storeCommandInScript(const char *cmd, const char *line)
 	char *path;
 	FILE *f;
 	char buffer[128];
+	mode_t permissions;
 
-	path = wstrconcat(wusergnusteppath(), "/Library/WindowMaker/autostart");
+	/* Calculate permission to be Executable but taking into account user's umask */
+	permissions = umask(0);
+	umask(permissions);
+	permissions = (S_IRWXU | S_IRWXG | S_IRWXO) & (~permissions);
+
+	path = wstrconcat(wuserdatapath(), "/" PACKAGE_TARNAME "/autostart");
 
 	f = fopen(path, "rb");
 	if (!f) {
@@ -670,7 +665,7 @@ static void storeCommandInScript(const char *cmd, const char *line)
 		char *tmppath;
 		FILE *fo;
 
-		tmppath = wstrconcat(wusergnusteppath(), "/Library/WindowMaker/autostart.tmp");
+		tmppath = wstrconcat(wuserdatapath(), "/" PACKAGE_TARNAME "/autostart.tmp");
 		fo = fopen(tmppath, "wb");
 		if (!fo) {
 			werror(_("could not create temporary file %s"), tmppath);
@@ -710,8 +705,8 @@ static void storeCommandInScript(const char *cmd, const char *line)
 		}
 		wfree(tmppath);
 	}
-	sprintf(buffer, "chmod u+x %s", path);
-	system(buffer);
+	if (chmod(path, permissions) != 0)
+		wwarning(_("could not set permission 0%03o on file \"%s\""), permissions, path);
 
  end:
 	wfree(path);
@@ -726,8 +721,6 @@ static void storeData(_Panel * panel)
 	char buffer[64];
 	int i;
 	char *tmp, *p;
-	static char *button[4] = { "None", "OpenApplicationsMenu", "OpenWindowListMenu", "SelectWindows" };
-	static char *wheel[2] = { "None", "SwitchWorkspaces" };
 	WMUserDefaults *udb = WMGetStandardUserDefaults();
 
 	if (!WMGetUDBoolForKey(udb, "NoXSetStuff")) {
@@ -746,32 +739,36 @@ static void storeData(_Panel * panel)
 	tmp = WMGetTextFieldText(panel->ddelaT);
 	if (sscanf(tmp, "%i", &i) == 1 && i > 0)
 		SetIntegerForKey(i, "DoubleClickTime");
+	wfree(tmp);
 
 	SetBoolForKey(WMGetButtonSelected(panel->disaB), "DisableWSMouseActions");
 
-	i = WMGetPopUpButtonSelectedItem(panel->button1P);
-	SetStringForKey(button[i], "MouseLeftButtonAction");
+	for (i = 0; i < wlengthof(button_list); i++) {
+		const char *db_value;
+		int action;
 
-	i = WMGetPopUpButtonSelectedItem(panel->button2P);
-	SetStringForKey(button[i], "MouseMiddleButtonAction");
-
-	i = WMGetPopUpButtonSelectedItem(panel->button3P);
-	SetStringForKey(button[i], "MouseRightButtonAction");
-
-	i = WMGetPopUpButtonSelectedItem(panel->wheelP);
-	SetStringForKey(wheel[i], "MouseWheelAction");
+		action = WMGetPopUpButtonSelectedItem(panel->mouse_action[i].popup);
+		if (action < 0)
+			continue;
+		if (button_list[i].type == T_BUTTON)
+			db_value = button_actions[action].db_value;
+		else
+			db_value = wheel_actions[action].db_value;
+		SetStringForKey(db_value, button_list[i].db_key);
+	}
 
 	tmp = WMGetPopUpButtonItem(panel->grabP, WMGetPopUpButtonSelectedItem(panel->grabP));
 	tmp = wstrdup(tmp);
 	p = strchr(tmp, ' ');
-	*p = 0;
+	if (p != NULL)
+		*p = '\0';
 
 	SetStringForKey(tmp, "ModifierKey");
 
 	wfree(tmp);
 }
 
-Panel *InitMouseSettings(WMScreen * scr, WMWidget * parent)
+Panel *InitMouseSettings(WMWidget *parent)
 {
 	_Panel *panel;
 
@@ -784,14 +781,6 @@ Panel *InitMouseSettings(WMScreen * scr, WMWidget * parent)
 	modifierNames[6] = wstrdup(_("Mod4"));
 	modifierNames[7] = wstrdup(_("Mod5"));
 
-	buttonActions[0] = wstrdup(_("None"));
-	buttonActions[1] = wstrdup(_("Applications Menu"));
-	buttonActions[2] = wstrdup(_("Window List Menu"));
-	buttonActions[3] = wstrdup(_("Select Windows"));
-
-	wheelActions[0] = wstrdup(_("None"));
-	wheelActions[1] = wstrdup(_("Switch Workspaces"));
-
 	panel = wmalloc(sizeof(_Panel));
 
 	panel->sectionName = _("Mouse Preferences");
