@@ -51,7 +51,7 @@
 /**** Global varianebles ****/
 
 #define MOD_MASK wPreferences.modifier_mask
-#define CACHE_ICON_PATH "/Library/WindowMaker/CachedPixmaps"
+#define CACHE_ICON_PATH "/" PACKAGE_TARNAME "/CachedPixmaps"
 #define ICON_BORDER 3
 
 static void miniwindowExpose(WObjDescriptor *desc, XEvent *event);
@@ -422,19 +422,17 @@ static char *get_icon_cache_path(void)
 {
 	const char *prefix;
 	char *path;
-	int len, ret;
+	int ret;
 
-	prefix = wusergnusteppath();
-	len = strlen(prefix) + strlen(CACHE_ICON_PATH) + 2;
-	path = wmalloc(len);
-	snprintf(path, len, "%s%s/", prefix, CACHE_ICON_PATH);
+	prefix = wuserdatapath();
+	path = wstrconcat(prefix, CACHE_ICON_PATH "/");
 
 	/* If the folder exists, exit */
 	if (access(path, F_OK) == 0)
 		return path;
 
 	/* Create the folder */
-	ret = wmkdirhier((const char *) path);
+	ret = wmkdirhier(path);
 
 	/* Exit 1 on success, 0 on failure */
 	if (ret == 1)
@@ -470,7 +468,7 @@ static RImage *get_wwindow_image_from_wmhints(WWindow *wwin, WIcon *icon)
  */
 char *wIconStore(WIcon *icon)
 {
-	char *path, *dir_path, *file;
+	char *path, *dir_path, *file, *filename;
 	int len = 0;
 	RImage *image = NULL;
 	WWindow *wwin = icon->owner;
@@ -488,15 +486,23 @@ char *wIconStore(WIcon *icon)
 		return NULL;
 	}
 
-	len = strlen(dir_path) + strlen(file) + 5;
-	path = wmalloc(len);
-	snprintf(path, len, "%s%s.xpm", dir_path, file);
-	wfree(dir_path);
+	/* Create the file name */
+	len = strlen(file) + 5;
+	filename = wmalloc(len);
+	snprintf(filename, len, "%s.xpm", file);
 	wfree(file);
 
+	/* Create the full path, including the filename */
+	len = strlen(dir_path) + strlen(filename) + 1;
+	path = wmalloc(len);
+	snprintf(path, len, "%s%s", dir_path, filename);
+	wfree(dir_path);
+
 	/* If icon exists, exit */
-	if (access(path, F_OK) == 0)
-		return path;
+	if (access(path, F_OK) == 0) {
+		wfree(path);
+		return filename;
+	}
 
 	if (wwin->net_icon_image)
 		image = RRetainImage(wwin->net_icon_image);
@@ -505,17 +511,37 @@ char *wIconStore(WIcon *icon)
 
 	if (!image) {
 		wfree(path);
+		wfree(filename);
 		return NULL;
 	}
 
 	if (!RSaveImage(image, path, "XPM")) {
-		wfree(path);
-		path = NULL;
+		wfree(filename);
+		filename = NULL;
 	}
 
+	wfree(path);
 	RReleaseImage(image);
 
-	return path;
+	return filename;
+}
+
+void remove_cache_icon(char *filename)
+{
+	char *cachepath;
+
+	if (!filename)
+		return;
+
+	cachepath = get_icon_cache_path();
+	if (!cachepath)
+		return;
+
+	/* Filename check/parse could be here */
+	if (!strncmp(filename, cachepath, strlen(cachepath)))
+		unlink(filename);
+
+	wfree(cachepath);
 }
 
 static void cycleColor(void *data)
@@ -640,7 +666,7 @@ void update_icon_pixmap(WIcon *icon)
 		XFreePixmap(dpy, icon->pixmap);
 
 	icon->pixmap = None;
- 
+
 	/* Create the pixmap */
 	if (icon->file_image)
 		icon_update_pixmap(icon, icon->file_image);
@@ -748,7 +774,12 @@ RImage *get_rimage_icon_from_wm_hints(WIcon *icon)
 {
 	RImage *image = NULL;
 	unsigned int w, h, d;
-	WWindow *wwin = icon->owner;
+	WWindow *wwin;
+
+	if ((!icon) || (!icon->owner))
+		return NULL;
+
+	wwin = icon->owner;
 
 	if (!getSize(wwin->wm_hints->icon_pixmap, &w, &h, &d)) {
 		icon->owner->wm_hints->flags &= ~IconPixmapHint;
