@@ -24,17 +24,18 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <time.h>
+#include <errno.h>
+#include <sys/stat.h>
+#include <unistd.h>
 
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/Xatom.h>
-#ifdef SHAPE
-#include <X11/extensions/shape.h>
-#endif
 #ifdef KEEP_XKB_LOCK_STATUS
 #include <X11/XKBlib.h>
 #endif				/* KEEP_XKB_LOCK_STATUS */
-#ifdef HAVE_XRANDR
+#ifdef USE_RANDR
 #include <X11/extensions/Xrandr.h>
 #endif
 
@@ -57,10 +58,12 @@
 #include "geomview.h"
 #include "wmspec.h"
 #include "rootmenu.h"
+#include "misc.h"
 
 #include "xinerama.h"
 
 #include <WINGs/WUtil.h>
+#include <WINGs/WINGsP.h>
 
 #include "defaults.h"
 
@@ -69,25 +72,10 @@
     |SubstructureRedirectMask|ButtonPressMask|ButtonReleaseMask\
     |KeyPressMask|KeyReleaseMask)
 
-/**** Global variables ****/
-extern Cursor wCursor[WCUR_LAST];
-extern Atom _XA_WINDOWMAKER_STATE;
-extern Atom _XA_WINDOWMAKER_NOTICEBOARD;
-
-extern int wScreenCount;
-
-#ifdef KEEP_XKB_LOCK_STATUS
-extern int wXkbSupported;
+#ifdef USE_ICCCM_WMREPLACE
+#define REPLACE_WM_TIMEOUT 15
 #endif
 
-#ifdef HAVE_XRANDR
-Bool has_randr;
-int randr_event_base;
-#endif
-
-extern WDDomain *WDWindowMaker;
-
-/**** Local ****/
 #define STIPPLE_WIDTH 2
 #define STIPPLE_HEIGHT 2
 static char STIPPLE_DATA[] = { 0x02, 0x01 };
@@ -112,6 +100,110 @@ static void make_keys(void)
 	dDrawers = WMCreatePLString("Drawers");
 }
 
+/*
+ * Support for ICCCM 2.0: Window Manager Replacement protocol
+ * See: http://www.x.org/releases/X11R7.6/doc/xorg-docs/specs/ICCCM/icccm.html
+ *
+ * Basically, user should be able to dynamically change its window manager; this is done
+ * cooperatively through a special Selection ("WM_Sn" where 'n' is the X screen number)
+ *
+ * This function does 2 things:
+ *  - it checks if this selection is already owned, which means that another window
+ * manager is running. If it is the case and user specified '--replace' on the command
+ * line, then it asks the WM to shut down;
+ *  - when ok, it sets the selection ownership to ourself, so another window manager
+ * may ask us to shut down (this is handled in "event.c")
+ */
+static Bool replace_existing_wm(WScreen *scr)
+{
+#ifdef USE_ICCCM_WMREPLACE
+	char atomName[16];
+	Window wm;
+	XSetWindowAttributes attribs;
+	XClientMessageEvent event;
+	unsigned long current_time;
+	int ret;
+
+	/* Try to acquire the atom named WM_S<screen> */
+	ret = snprintf(atomName, sizeof(atomName), "WM_S%d", scr->screen);
+	if (ret < 0 || ret == sizeof(atomName)) {
+		werror("out of memory trying to allocate window manager selection atom for screen %d", scr->screen);
+		return False;
+	}
+
+	scr->sn_atom = XInternAtom(dpy, atomName, False);
+	if (! scr->sn_atom)
+		return False;
+
+	/* Check if an existing window manager owns the selection */
+	wm = XGetSelectionOwner(dpy, scr->sn_atom);
+	if (wm) {
+		if (!wPreferences.flags.replace) {
+			wmessage(_("another window manager is running"));
+			wwarning(_("use the --replace flag to replace it"));
+			return False;
+		}
+
+		attribs.event_mask = StructureNotifyMask;
+		if (!XChangeWindowAttributes(dpy, wm, CWEventMask, &attribs))
+			wm = None;
+	}
+#endif
+
+	/* for our window manager info notice board and the selection owner */
+	scr->info_window = XCreateSimpleWindow(dpy, scr->root_win, 0, 0, 10, 10, 0, 0, 0);
+
+#ifdef USE_ICCCM_WMREPLACE
+	/* Try to acquire the selection */
+	current_time = CurrentTime;
+	ret = XSetSelectionOwner(dpy, scr->sn_atom, scr->info_window, current_time);
+	if (ret == BadAtom || ret == BadWindow)
+		return False;
+
+	/* Wait for other window manager to exit */
+	if (wm) {
+		unsigned long wait = 0;
+		unsigned long timeout = REPLACE_WM_TIMEOUT * 1000000L;
+		XEvent event;
+
+		while (wait < timeout) {
+			if (!(wait % 1000000))
+				wmessage(_("waiting %lus for other window manager to exit"), (timeout - wait) / 1000000L);
+
+			if (XCheckWindowEvent(dpy, wm, StructureNotifyMask, &event))
+				if (event.type == DestroyNotify)
+					break;
+
+			wusleep(100000);
+			wait += 100000;
+		}
+
+		if (wait >= timeout) {
+			wwarning(_("other window manager hasn't exited!"));
+			return False;
+		}
+
+		wmessage(_("replacing the other window manager"));
+	}
+
+	if (XGetSelectionOwner(dpy, scr->sn_atom) != scr->info_window)
+		return False;
+
+	event.type = ClientMessage;
+	event.message_type = scr->sn_atom;
+	event.format = 32;
+	event.data.l[0] = (long) current_time;
+	event.data.l[1] = (long) scr->sn_atom;
+	event.data.l[2] = (long) scr->info_window;
+	event.data.l[3] = (long) 0L;
+	event.data.l[4] = (long) 0L;
+	event.window = scr->root_win;
+	XSendEvent(dpy, scr->root_win, False, StructureNotifyMask, (XEvent *) &event);
+#endif
+
+	return True;
+}
+
 /*
  *----------------------------------------------------------------------
  * alreadyRunningError--
@@ -128,6 +220,10 @@ static void make_keys(void)
  */
 static int alreadyRunningError(Display * dpy, XErrorEvent * error)
 {
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) dpy;
+	(void) error;
+
 	CantManageScreen = 1;
 	return -1;
 }
@@ -223,14 +319,34 @@ static void allocButtonPixmaps(WScreen * scr)
 	scr->b_pixmaps[WBUT_KILL] = pix;
 }
 
+static int get_dot_mult(void)
+{
+    int dot_mult;
+    
+    /*
+     * Since it's an int, the multiplier will grow only when
+     * a certain icon size is reached (1 for 64, 2 for 128,
+     * 3 for 192, and so on).
+     */
+    dot_mult = wPreferences.icon_size / 64;
+
+    if (dot_mult < 1)
+        dot_mult = 1;
+    
+    return dot_mult;
+}
+
 static void draw_dot(WScreen * scr, Drawable d, int x, int y, GC gc)
 {
+    int dot_mult;
+
+    dot_mult = get_dot_mult();
 	XSetForeground(dpy, gc, scr->black_pixel);
-	XDrawLine(dpy, d, gc, x, y, x + 1, y);
-	XDrawPoint(dpy, d, gc, x, y + 1);
+	XFillRectangle(dpy, d, gc, x, y, x + (2 * dot_mult), y + (1 * dot_mult));
+	XFillRectangle(dpy, d, gc, x, y + (1 * dot_mult), x + (1 * dot_mult), y + (2 * dot_mult));
 	XSetForeground(dpy, gc, scr->white_pixel);
-	XDrawLine(dpy, d, gc, x + 2, y, x + 2, y + 1);
-	XDrawPoint(dpy, d, gc, x + 1, y + 1);
+	XFillRectangle(dpy, d, gc, x + (2 * dot_mult), y, x + (3 * dot_mult), y + (2 * dot_mult));
+	XFillRectangle(dpy, d, gc, x + (1 * dot_mult), y + (1 * dot_mult), x + (2 * dot_mult), y + (2 * dot_mult));
 }
 
 static WPixmap *make3Dots(WScreen * scr)
@@ -239,15 +355,17 @@ static WPixmap *make3Dots(WScreen * scr)
 	GC gc2, gc;
 	XGCValues gcv;
 	Pixmap pix, mask;
+    int dot_mult;
 
+    dot_mult = get_dot_mult();
 	gc = scr->copy_gc;
 	pix = XCreatePixmap(dpy, scr->w_win, wPreferences.icon_size, wPreferences.icon_size, scr->w_depth);
 	XSetForeground(dpy, gc, scr->black_pixel);
 	XFillRectangle(dpy, pix, gc, 0, 0, wPreferences.icon_size, wPreferences.icon_size);
 	XSetForeground(dpy, gc, scr->white_pixel);
-	draw_dot(scr, pix, 4, wPreferences.icon_size - 6, gc);
-	draw_dot(scr, pix, 9, wPreferences.icon_size - 6, gc);
-	draw_dot(scr, pix, 14, wPreferences.icon_size - 6, gc);
+	draw_dot(scr, pix, 4 * dot_mult, wPreferences.icon_size - (6 * dot_mult), gc);
+	draw_dot(scr, pix, 9 * dot_mult, wPreferences.icon_size - (6 * dot_mult), gc);
+	draw_dot(scr, pix, 14 * dot_mult, wPreferences.icon_size - (6 * dot_mult), gc);
 
 	mask = XCreatePixmap(dpy, scr->w_win, wPreferences.icon_size, wPreferences.icon_size, 1);
 	gcv.foreground = 0;
@@ -255,13 +373,13 @@ static WPixmap *make3Dots(WScreen * scr)
 	gc2 = XCreateGC(dpy, mask, GCForeground | GCGraphicsExposures, &gcv);
 	XFillRectangle(dpy, mask, gc2, 0, 0, wPreferences.icon_size, wPreferences.icon_size);
 	XSetForeground(dpy, gc2, 1);
-	XFillRectangle(dpy, mask, gc2, 4, wPreferences.icon_size - 6, 3, 2);
-	XFillRectangle(dpy, mask, gc2, 9, wPreferences.icon_size - 6, 3, 2);
-	XFillRectangle(dpy, mask, gc2, 14, wPreferences.icon_size - 6, 3, 2);
+	XFillRectangle(dpy, mask, gc2, 4 * dot_mult, wPreferences.icon_size - (6 * dot_mult), 3 * dot_mult, 2 * dot_mult);
+	XFillRectangle(dpy, mask, gc2, 9 * dot_mult, wPreferences.icon_size - (6 * dot_mult), 3 * dot_mult, 2 * dot_mult);
+	XFillRectangle(dpy, mask, gc2, 14 * dot_mult, wPreferences.icon_size - (6 * dot_mult), 3 * dot_mult, 2 * dot_mult);
 
 	XFreeGC(dpy, gc2);
 
-	wpix = wPixmapCreate(scr, pix, mask);
+	wpix = wPixmapCreate(pix, mask);
 	wpix->shared = 1;
 
 	return wpix;
@@ -366,45 +484,34 @@ static void createPixmaps(WScreen * scr)
 	WPixmap *pix;
 
 	/* load pixmaps */
-	pix = wPixmapCreateFromXBMData(scr, (char *)MENU_RADIO_INDICATOR_XBM_DATA,
-				       (char *)MENU_RADIO_INDICATOR_XBM_DATA,
-				       MENU_RADIO_INDICATOR_XBM_SIZE,
-				       MENU_RADIO_INDICATOR_XBM_SIZE, scr->black_pixel, scr->white_pixel);
-	if (pix != NULL)
-		pix->shared = 1;
-	scr->menu_radio_indicator = pix;
+#define LOADPIXMAPINDICATOR(XBM,W,I) {\
+	pix = wPixmapCreateFromXBMData(scr, (char *)XBM, (char *)XBM,\
+					W, MENU_SNAP_INDICATOR_H_XBM_SIZE,\
+					scr->black_pixel, scr->white_pixel);\
+	if (pix != NULL)\
+		pix->shared = 1;\
+	scr->I = pix;}
 
-	pix = wPixmapCreateFromXBMData(scr, (char *)MENU_CHECK_INDICATOR_XBM_DATA,
-				       (char *)MENU_CHECK_INDICATOR_XBM_DATA,
-				       MENU_CHECK_INDICATOR_XBM_SIZE,
-				       MENU_CHECK_INDICATOR_XBM_SIZE, scr->black_pixel, scr->white_pixel);
-	if (pix != NULL)
-		pix->shared = 1;
-	scr->menu_check_indicator = pix;
+	LOADPIXMAPINDICATOR(MENU_RADIO_INDICATOR_XBM_DATA, MENU_RADIO_INDICATOR_XBM_SIZE, menu_radio_indicator)
+	LOADPIXMAPINDICATOR(MENU_CHECK_INDICATOR_XBM_DATA, MENU_CHECK_INDICATOR_XBM_SIZE, menu_check_indicator)
+	LOADPIXMAPINDICATOR(MENU_MINI_INDICATOR_XBM_DATA, MENU_MINI_INDICATOR_XBM_SIZE, menu_mini_indicator)
+	LOADPIXMAPINDICATOR(MENU_HIDE_INDICATOR_XBM_DATA, MENU_HIDE_INDICATOR_XBM_SIZE, menu_hide_indicator)
+	LOADPIXMAPINDICATOR(MENU_SHADE_INDICATOR_XBM_DATA, MENU_SHADE_INDICATOR_XBM_SIZE, menu_shade_indicator)
 
-	pix = wPixmapCreateFromXBMData(scr, (char *)MENU_MINI_INDICATOR_XBM_DATA,
-				       (char *)MENU_MINI_INDICATOR_XBM_DATA,
-				       MENU_MINI_INDICATOR_XBM_SIZE,
-				       MENU_MINI_INDICATOR_XBM_SIZE, scr->black_pixel, scr->white_pixel);
-	if (pix != NULL)
-		pix->shared = 1;
-	scr->menu_mini_indicator = pix;
+	LOADPIXMAPINDICATOR(MENU_SNAP_V_INDICATOR_XBM_DATA, MENU_SNAP_INDICATOR_W_XBM_SIZE, menu_snap_vertical_indicator)
+	LOADPIXMAPINDICATOR(MENU_SNAP_H_INDICATOR_XBM_DATA, MENU_SNAP_INDICATOR_W_XBM_SIZE, menu_snap_horizontal_indicator)
+	LOADPIXMAPINDICATOR(MENU_CENTRAL_INDICATOR_XBM_DATA, MENU_SNAP_INDICATOR_W_XBM_SIZE, menu_central_indicator)
+	LOADPIXMAPINDICATOR(MENU_SNAP_RH_INDICATOR_XBM_DATA, MENU_SNAP_INDICATOR_W_XBM_SIZE, menu_snap_rh_indicator)
+	LOADPIXMAPINDICATOR(MENU_SNAP_LH_INDICATOR_XBM_DATA, MENU_SNAP_INDICATOR_W_XBM_SIZE, menu_snap_lh_indicator)
+	LOADPIXMAPINDICATOR(MENU_SNAP_TH_INDICATOR_XBM_DATA, MENU_SNAP_INDICATOR_W_XBM_SIZE, menu_snap_th_indicator)
+	LOADPIXMAPINDICATOR(MENU_SNAP_BH_INDICATOR_XBM_DATA, MENU_SNAP_INDICATOR_W_XBM_SIZE, menu_snap_bh_indicator)
+	LOADPIXMAPINDICATOR(MENU_SNAP_TL_INDICATOR_XBM_DATA, MENU_SNAP_INDICATOR_W_XBM_SIZE, menu_snap_tl_indicator)
+	LOADPIXMAPINDICATOR(MENU_SNAP_TR_INDICATOR_XBM_DATA, MENU_SNAP_INDICATOR_W_XBM_SIZE, menu_snap_tr_indicator)
+	LOADPIXMAPINDICATOR(MENU_SNAP_BL_INDICATOR_XBM_DATA, MENU_SNAP_INDICATOR_W_XBM_SIZE, menu_snap_bl_indicator)
+	LOADPIXMAPINDICATOR(MENU_SNAP_BR_INDICATOR_XBM_DATA, MENU_SNAP_INDICATOR_W_XBM_SIZE, menu_snap_br_indicator)
+	LOADPIXMAPINDICATOR(MENU_SNAP_TILED_INDICATOR_XBM_DATA, MENU_SNAP_INDICATOR_W_XBM_SIZE, menu_snap_tiled_indicator)
 
-	pix = wPixmapCreateFromXBMData(scr, (char *)MENU_HIDE_INDICATOR_XBM_DATA,
-				       (char *)MENU_HIDE_INDICATOR_XBM_DATA,
-				       MENU_HIDE_INDICATOR_XBM_SIZE,
-				       MENU_HIDE_INDICATOR_XBM_SIZE, scr->black_pixel, scr->white_pixel);
-	if (pix != NULL)
-		pix->shared = 1;
-	scr->menu_hide_indicator = pix;
-
-	pix = wPixmapCreateFromXBMData(scr, (char *)MENU_SHADE_INDICATOR_XBM_DATA,
-				       (char *)MENU_SHADE_INDICATOR_XBM_DATA,
-				       MENU_SHADE_INDICATOR_XBM_SIZE,
-				       MENU_SHADE_INDICATOR_XBM_SIZE, scr->black_pixel, scr->white_pixel);
-	if (pix != NULL)
-		pix->shared = 1;
-	scr->menu_shade_indicator = pix;
+#undef LOADPIXMAPINDICATOR
 
 	create_logo_image(scr);
 
@@ -416,7 +523,7 @@ static void createPixmaps(WScreen * scr)
 
 void create_logo_image(WScreen *scr)
 {
-	RImage *image = get_icon_image(scr, "Logo", "WMPanel", wPreferences.icon_size);
+	RImage *image = get_icon_image(scr, "Logo", "WMPanel", 128);
 
 	if (!image) {
 		wwarning(_("could not load logo image for panels: %s"), RMessageForError(RErrorCode));
@@ -464,7 +571,7 @@ static void createInternalWindows(WScreen * scr)
 	attribs.override_redirect = True;
 	attribs.background_pixmap = None;
 	attribs.background_pixel = scr->white_pixel;
-	attribs.cursor = wCursor[WCUR_DEFAULT];
+	attribs.cursor = wPreferences.cursor[WCUR_NORMAL];
 	vmask |= CWColormap;
 	attribs.colormap = scr->w_colormap;
 	scr->dock_shadow =
@@ -481,6 +588,7 @@ static void createInternalWindows(WScreen * scr)
 	scr->workspace_name =
 	    XCreateWindow(dpy, scr->root_win, 0, 0, 10, 10, 0, scr->w_depth,
 			  CopyFromParent, scr->w_visual, vmask, &attribs);
+	scr->mini_screenshot_timeout = 0;
 }
 
 /*
@@ -505,7 +613,7 @@ WScreen *wScreenInit(int screen_number)
 {
 	WScreen *scr;
 	XIconSize icon_size[1];
-	RContextAttributes rattr;
+	RContextAttributes rattr = {};
 	long event_mask;
 	XErrorHandler oldHandler;
 	int i;
@@ -539,27 +647,29 @@ WScreen *wScreenInit(int screen_number)
 	scr->fakeGroupLeaders = WMCreateArray(16);
 
 	CantManageScreen = 0;
-	oldHandler = XSetErrorHandler((XErrorHandler) alreadyRunningError);
+	oldHandler = XSetErrorHandler(alreadyRunningError);
 
-	event_mask = EVENT_MASK;
-
-	if (wPreferences.disable_root_mouse) {
-		event_mask &= ~(ButtonPressMask | ButtonReleaseMask);
+	/* Do we need to replace an existing window manager? */
+	if (!replace_existing_wm(scr)) {
+		XDestroyWindow(dpy, scr->info_window);
+		wfree(scr);
+		return NULL;
 	}
 
+	event_mask = EVENT_MASK;
 	XSelectInput(dpy, scr->root_win, event_mask);
 
 #ifdef KEEP_XKB_LOCK_STATUS
 	/* Only GroupLock doesn't work correctly in my system since right-alt
 	 * can change mode while holding it too - ]d
 	 */
-	if (wXkbSupported) {
+	if (w_global.xext.xkb.supported) {
 		XkbSelectEvents(dpy, XkbUseCoreKbd, XkbStateNotifyMask, XkbStateNotifyMask);
 	}
 #endif				/* KEEP_XKB_LOCK_STATUS */
 
-#ifdef HAVE_XRANDR
-	if (has_randr)
+#ifdef USE_RANDR
+	if (w_global.xext.randr.supported)
 		XRRSelectInput(dpy, scr->root_win, RRScreenChangeNotifyMask);
 #endif
 
@@ -571,7 +681,7 @@ WScreen *wScreenInit(int screen_number)
 		return NULL;
 	}
 
-	XDefineCursor(dpy, scr->root_win, wCursor[WCUR_ROOT]);
+	XDefineCursor(dpy, scr->root_win, wPreferences.cursor[WCUR_ROOT]);
 
 	/* screen descriptor for raster graphic library */
 	rattr.flags = RC_RenderMode | RC_ColorsPerChannel | RC_StandardColormap;
@@ -600,23 +710,10 @@ WScreen *wScreenInit(int screen_number)
 
 		scr->rcontext = RCreateContext(dpy, screen_number, &rattr);
 	}
-
-	if (!scr->rcontext) {
-		wwarning(_("could not initialize graphics library context: %s"), RMessageForError(RErrorCode));
-		wAbort(False);
-	} else {
-		char **formats;
-		int i = 0;
-
-		formats = RSupportedFileFormats();
-		if (formats) {
-			for (i = 0; formats[i] != NULL; i++) {
-				if (strcmp(formats[i], "TIFF") == 0) {
-					scr->flags.supports_tiff = 1;
-					break;
-				}
-			}
-		}
+	if (scr->rcontext == NULL) {
+		wfatal(_("can't create Context on screen %d, %s"),
+		       screen_number, RMessageForError(RErrorCode));
+		goto abort_no_context;
 	}
 
 	scr->w_win = scr->rcontext->drawable;
@@ -629,6 +726,13 @@ WScreen *wScreenInit(int screen_number)
 
 	if (!scr->wmscreen) {
 		wfatal(_("could not initialize WINGs widget set"));
+		RDestroyContext(scr->rcontext);
+	abort_no_context:
+		WMFreeArray(scr->fakeGroupLeaders);
+		wfree(scr->totalUsableArea);
+		wfree(scr->usableArea);
+		WMFreeBag(scr->stacking_list);
+		wfree(scr);
 		return NULL;
 	}
 
@@ -645,19 +749,16 @@ WScreen *wScreenInit(int screen_number)
 	/* create GCs with default values */
 	allocGCs(scr);
 
-	/* for our window manager info notice board. Need to
-	 * create before reading the defaults, because it will be used there.
-	 */
-	scr->info_window = XCreateSimpleWindow(dpy, scr->root_win, 0, 0, 10, 10, 0, 0, 0);
-
 	/* read defaults for this screen */
-	wReadDefaults(scr, WDWindowMaker->dictionary);
+	wReadDefaults(scr, w_global.domain.wmaker->dictionary);
 
 	{
 		XColor xcol;
 		/* frame boder color */
 		wGetColor(scr, WMGetColorRGBDescription(scr->frame_border_color), &xcol);
 		scr->frame_border_pixel = xcol.pixel;
+		wGetColor(scr, WMGetColorRGBDescription(scr->frame_focused_border_color), &xcol);
+		scr->frame_focused_border_pixel = xcol.pixel;
 		wGetColor(scr, WMGetColorRGBDescription(scr->frame_selected_border_color), &xcol);
 		scr->frame_selected_border_pixel = xcol.pixel;
 	}
@@ -685,9 +786,9 @@ WScreen *wScreenInit(int screen_number)
 	PropSetWMakerProtocols(scr->root_win);
 
 	/* setup our noticeboard */
-	XChangeProperty(dpy, scr->info_window, _XA_WINDOWMAKER_NOTICEBOARD,
+	XChangeProperty(dpy, scr->info_window, w_global.atom.wmaker.noticeboard,
 			XA_WINDOW, 32, PropModeReplace, (unsigned char *)&scr->info_window, 1);
-	XChangeProperty(dpy, scr->root_win, _XA_WINDOWMAKER_NOTICEBOARD,
+	XChangeProperty(dpy, scr->root_win, w_global.atom.wmaker.noticeboard,
 			XA_WINDOW, 32, PropModeReplace, (unsigned char *)&scr->info_window, 1);
 
 #ifdef BALLOON_TEXT
@@ -709,7 +810,7 @@ WScreen *wScreenInit(int screen_number)
 	return scr;
 }
 
-void wScreenUpdateUsableArea(WScreen * scr)
+void wScreenUpdateUsableArea(WScreen *scr)
 {
 	/*
 	 * scr->totalUsableArea[] will become the usableArea used for Windowplacement,
@@ -718,23 +819,10 @@ void wScreenUpdateUsableArea(WScreen * scr)
 	 */
 
 	WArea area;
-	int i, dock_head;
-	unsigned long best_area, tmp_area;
-	unsigned int size, position;
-
-	dock_head = scr->xine_info.primary_head;
-	best_area = 0;
-	size = wPreferences.workspace_border_size;
-	position = wPreferences.workspace_border_position;
-
-	if (scr->dock) {
-		WMRect rect;
-		rect.pos.x = scr->dock->x_pos;
-		rect.pos.y = scr->dock->y_pos;
-		rect.size.width = wPreferences.icon_size;
-		rect.size.height = wPreferences.icon_size;
-		dock_head = wGetHeadForRect(scr, rect);
-	}
+	int i;
+	unsigned long tmp_area, best_area = 0;
+	unsigned int size = wPreferences.workspace_border_size;
+	unsigned int position = wPreferences.workspace_border_position;
 
 	for (i = 0; i < wXineramaHeads(scr); ++i) {
 		WMRect rect = wGetRectForHead(scr, i);
@@ -743,16 +831,6 @@ void wScreenUpdateUsableArea(WScreen * scr)
 		scr->totalUsableArea[i].x2 = rect.pos.x + rect.size.width;
 		scr->totalUsableArea[i].y2 = rect.pos.y + rect.size.height;
 
-		if (scr->dock && dock_head == i && (!scr->dock->lowered || wPreferences.no_window_over_dock)) {
-			int offset = wPreferences.icon_size + DOCK_EXTRA_SPACE;
-
-			if (scr->dock->on_right_side) {
-				scr->totalUsableArea[i].x2 -= offset;
-			} else {
-				scr->totalUsableArea[i].x1 += offset;
-			}
-		}
-
 		if (wNETWMGetUsableArea(scr, i, &area)) {
 			scr->totalUsableArea[i].x1 = WMAX(scr->totalUsableArea[i].x1, area.x1);
 			scr->totalUsableArea[i].y1 = WMAX(scr->totalUsableArea[i].y1, area.y1);
@@ -762,23 +840,17 @@ void wScreenUpdateUsableArea(WScreen * scr)
 
 		scr->usableArea[i] = scr->totalUsableArea[i];
 
-#if 0
-		printf("usableArea[%d]: %d %d %d %d\n", i,
-		       scr->usableArea[i].x1, scr->usableArea[i].x2, scr->usableArea[i].y1, scr->usableArea[i].y2);
-#endif
 		if (wPreferences.no_window_over_icons) {
 			if (wPreferences.icon_yard & IY_VERT) {
-				if (wPreferences.icon_yard & IY_RIGHT) {
+				if (wPreferences.icon_yard & IY_RIGHT)
 					scr->totalUsableArea[i].x2 -= wPreferences.icon_size;
-				} else {
+				else
 					scr->totalUsableArea[i].x1 += wPreferences.icon_size;
-				}
 			} else {
-				if (wPreferences.icon_yard & IY_TOP) {
+				if (wPreferences.icon_yard & IY_TOP)
 					scr->totalUsableArea[i].y1 += wPreferences.icon_size;
-				} else {
+				else
 					scr->totalUsableArea[i].y2 -= wPreferences.icon_size;
-				}
 			}
 		}
 
@@ -826,7 +898,7 @@ void wScreenRestoreState(WScreen * scr)
 
 	make_keys();
 
-	if (wScreenCount == 1) {
+	if (w_global.screen_count == 1) {
 		path = wdefaultspathfordomain("WMState");
 	} else {
 		char buf[16];
@@ -835,34 +907,38 @@ void wScreenRestoreState(WScreen * scr)
 	}
 	scr->session_state = WMReadPropListFromFile(path);
 	wfree(path);
-	if (!scr->session_state && wScreenCount > 1) {
+	if (!scr->session_state && w_global.screen_count > 1) {
 		path = wdefaultspathfordomain("WMState");
 		scr->session_state = WMReadPropListFromFile(path);
 		wfree(path);
 	}
 
-	if (!scr->session_state) {
+	if (!scr->session_state)
 		scr->session_state = WMCreatePLDictionary(NULL, NULL);
-	}
 
 	if (!wPreferences.flags.nodock) {
 		state = WMGetFromPLDictionary(scr->session_state, dDock);
 		scr->dock = wDockRestoreState(scr, state, WM_DOCK);
-		/* If clip_merged_in_dock, setting scr->clip_icon is done by
-		 * wDockRestoreState()->wDockCreate()->mainIconCreate() */
 	}
 
 	if (!wPreferences.flags.noclip) {
 		state = WMGetFromPLDictionary(scr->session_state, dClip);
 		scr->clip_icon = wClipRestoreState(scr, state);
 	}
-	
+
 	if (!wPreferences.flags.nodrawer) {
+		if (!scr->dock->on_right_side) {
+			/* Drawer tile was created early in wScreenInit() -> wReadDefaults(). At
+			 * that time, scr->dock was NULL and the tile was created as if we were on
+			 * the right side. If we aren't, redo it now. */
+			assert(scr->drawer_tile);
+			RReleaseImage(scr->drawer_tile);
+			scr->drawer_tile = wDrawerMakeTile(scr, scr->icon_tile);
+		}
 		wDrawersRestoreState(scr);
 	}
 
 	wWorkspaceRestoreState(scr);
-
 	wScreenUpdateUsableArea(scr);
 }
 
@@ -893,16 +969,16 @@ void wScreenSaveState(WScreen * scr)
 	if (!wPreferences.flags.nodock) {
 		wDockSaveState(scr, old_state);
 	} else {
-		if ((foo = WMGetFromPLDictionary(old_state, dDock)) != NULL) {
+		foo = WMGetFromPLDictionary(old_state, dDock);
+		if (foo != NULL)
 			WMPutInPLDictionary(scr->session_state, dDock, foo);
-		}
 	}
 	if (!wPreferences.flags.noclip) {
 		wClipSaveState(scr);
 	} else {
-		if ((foo = WMGetFromPLDictionary(old_state, dClip)) != NULL) {
+		foo = WMGetFromPLDictionary(old_state, dClip);
+		if (foo != NULL)
 			WMPutInPLDictionary(scr->session_state, dClip, foo);
-		}
 	}
 
 	wWorkspaceSaveState(scr, old_state);
@@ -910,21 +986,22 @@ void wScreenSaveState(WScreen * scr)
 	if (!wPreferences.flags.nodrawer) {
 		wDrawersSaveState(scr);
 	} else {
-		if ((foo = WMGetFromPLDictionary(old_state, dDrawers)) != NULL) {
+		foo = WMGetFromPLDictionary(old_state, dDrawers);
+		if (foo != NULL)
 			WMPutInPLDictionary(scr->session_state, dDrawers, foo);
-		}
 	}
 
 
 	if (wPreferences.save_session_on_exit) {
 		wSessionSaveState(scr);
 	} else {
-		if ((foo = WMGetFromPLDictionary(old_state, dApplications)) != NULL) {
+		foo = WMGetFromPLDictionary(old_state, dApplications);
+		if (foo != NULL)
 			WMPutInPLDictionary(scr->session_state, dApplications, foo);
-		}
-		if ((foo = WMGetFromPLDictionary(old_state, dWorkspace)) != NULL) {
+
+		foo = WMGetFromPLDictionary(old_state, dWorkspace);
+		if (foo != NULL)
 			WMPutInPLDictionary(scr->session_state, dWorkspace, foo);
-		}
 	}
 
 	/* clean up */
@@ -932,7 +1009,7 @@ void wScreenSaveState(WScreen * scr)
 
 	wMenuSaveState(scr);
 
-	if (wScreenCount == 1) {
+	if (w_global.screen_count == 1) {
 		str = wdefaultspathfordomain("WMState");
 	} else {
 		char buf[16];
@@ -1045,3 +1122,246 @@ int wScreenKeepInside(WScreen * scr, int *x, int *y, int width, int height)
 
 	return moved;
 }
+
+static XImage *imageCaptureArea(WScreen *scr)
+{
+	XEvent event;
+	int quit = 0;
+	int xp = -1;
+	int yp = -1;
+	int w = 0, h = 0;
+	int x = xp, y = yp;
+
+	if (XGrabPointer(dpy, scr->root_win, False, ButtonMotionMask
+			| ButtonReleaseMask | ButtonPressMask, GrabModeAsync,
+			GrabModeAsync, None, wPreferences.cursor[WCUR_CAPTURE], CurrentTime) != Success) {
+		return NULL;
+	}
+
+	XGrabServer(dpy);
+
+	while (!quit) {
+		WMMaskEvent(dpy, ButtonReleaseMask | PointerMotionMask | ButtonPressMask | KeyPressMask, &event);
+
+		switch (event.type) {
+		case ButtonPress:
+			if (event.xbutton.button == Button1) {
+				xp = event.xbutton.x_root;
+				yp = event.xbutton.y_root;
+			}
+			break;
+		case ButtonRelease:
+			if (event.xbutton.button == Button1) {
+				quit = 1;
+				if (w > 0 && h > 0) {
+					XDrawRectangle(dpy, scr->root_win, scr->frame_gc, x, y, w, h);
+					XUngrabServer(dpy);
+					XUngrabPointer(dpy, CurrentTime);
+					return XGetImage(dpy, scr->root_win, x, y, w, h, AllPlanes, ZPixmap);
+				}
+			}
+			break;
+		case MotionNotify:
+			XDrawRectangle(dpy, scr->root_win, scr->frame_gc, x, y, w, h);
+			x = event.xmotion.x_root;
+                        if (x < xp) {
+                                w = xp - x;
+                        } else {
+                                w = x - xp;
+                                x = xp;
+                        }
+                        y = event.xmotion.y_root;
+                        if (y < yp) {
+                                h = yp - y;
+                        } else {
+                                h = y - yp;
+                                y = yp;
+                        }
+			XDrawRectangle(dpy, scr->root_win, scr->frame_gc, x, y, w, h);
+			break;
+		case KeyPress:
+			if (W_KeycodeToKeysym(dpy, event.xkey.keycode, 0) == XK_Escape)
+				quit = 1;
+			break;
+		default:
+			WMHandleEvent(&event);
+			break;
+		}
+	}
+
+	XUngrabServer(dpy);
+	XUngrabPointer(dpy, CurrentTime);
+	return NULL;
+}
+
+static void hideMiniScreenshot(void *data)
+{
+	WScreen *scr = (WScreen *) data;
+
+	if (time(NULL) < scr->mini_screenshot_timeout) {
+		scr->mini_screenshot_timer = WMAddTimerHandler(WORKSPACE_NAME_FADE_DELAY, hideMiniScreenshot, scr);
+	} else {
+		XWindowAttributes attr;
+
+		WMDeleteTimerHandler(scr->mini_screenshot_timer);
+		if (XGetWindowAttributes(dpy, scr->mini_screenshot, &attr))
+			slide_window(scr->mini_screenshot, attr.x, attr.y, attr.x + attr.width, attr.y);
+		XUnmapWindow(dpy, scr->mini_screenshot);
+		XDestroyWindow(dpy, scr->mini_screenshot);
+		scr->mini_screenshot_timeout = 0;
+	}
+}
+
+static void showMiniScreenshot(WScreen *scr, RImage *img)
+{
+	Pixmap pix;
+	int x = scr->scr_width - img->width - 20;
+	int y = scr->scr_height - img->height - 20;
+
+	if (!scr->mini_screenshot_timeout) {
+		Window win;
+
+		win = XCreateSimpleWindow(dpy, scr->root_win, x, y,
+			img->width, img->height, scr->frame_border_width, 0, 0);
+		scr->mini_screenshot = win;
+	}
+	RConvertImage(scr->rcontext, img, &pix);
+	XMapWindow(dpy, scr->mini_screenshot);
+	XCopyArea(dpy, pix, scr->mini_screenshot, scr->rcontext->copy_gc, 0, 0, img->width, img->height, 0, 0);
+	XFlush(dpy);
+
+	scr->mini_screenshot_timeout = time(NULL) + 2;
+	scr->mini_screenshot_timer = WMAddTimerHandler(WORKSPACE_NAME_FADE_DELAY, hideMiniScreenshot, scr);
+}
+
+void ScreenCapture(WScreen *scr, int mode)
+{
+	time_t s;
+	short i = 0;
+	struct tm *tm_info;
+	char index_str[12] = "";
+	char filename_date_part[60];
+	char filename[60];
+	char *filepath;
+	char *screenshot_dir;
+	RImage *img = NULL;
+	RImage *scale_img = NULL;
+
+#ifdef USE_PNG
+	char *filetype = ".png";
+#else
+#ifdef USE_JPEG
+	char *filetype = ".jpg";
+#else
+	char *filetype = NULL;
+#endif
+#endif
+
+	if (!filetype) {
+		werror(_("Unable to find a proper screenshot image format"));
+		return;
+	}
+
+	screenshot_dir = wstrconcat(wusergnusteppath(), "/Library/WindowMaker/Screenshots/");
+	if (-1 == mkdir(screenshot_dir, 0700) && errno != EEXIST) {
+		wfree(screenshot_dir);
+		werror(_("Unable to create screenshot directory: %s"), strerror(errno));
+		return;
+	}
+
+	s = time(NULL);
+	tm_info = localtime(&s);
+	strftime(filename_date_part, sizeof(filename_date_part), "screenshot_%Y-%m-%d_at_%H:%M:%S", tm_info);
+	strcpy(filename, filename_date_part);
+
+	filepath = wstrconcat(screenshot_dir, strcat(filename, filetype));
+	while (access(filepath, F_OK) == 0 && i < 600) {
+		i++;
+		strcpy(filename, filename_date_part);
+		sprintf(index_str, "_%d", i);
+		strncat(filename, index_str, sizeof(filename) - strlen(filename) - 1);
+		wfree(filepath);
+		filepath = wstrconcat(screenshot_dir, strcat(filename, filetype));
+	}
+
+	/* cannot generate an available filename ?! */
+	if (i == 600) {
+		wfree(filepath);
+		wfree(screenshot_dir);
+		werror(_("Could not generate a free screenshot filename"));
+		return;
+	}
+
+	switch (mode) {
+		WWindow *wwin;
+		XImage *pimg;
+
+		case PRINT_WINDOW:
+			wwin = scr->focused_window;
+			if (wwin && !wwin->flags.shaded) {
+				/*
+				 * check if hint WM_TAKE_FOCUS is set, if it's the case
+				 * we can take screenshot of the out of screen window
+				 */
+				if (wwin->focus_mode >= WFM_LOCALLY_ACTIVE) {
+					img = RCreateImageFromDrawable(scr->rcontext, wwin->client_win, None);
+				}
+				else {
+					/* we will only capture the visible window part */
+					int x_crop = 0;
+					int y_crop = 0;
+					int w_crop = wwin->client.width;
+					int h_crop = wwin->client.height;
+
+					if (wwin->client.x > 0)
+						x_crop = wwin->client.x;
+					if (wwin->client.y > 0)
+						y_crop = wwin->client.y;
+
+					if (wwin->client.x + wwin->client.width > scr->scr_width)
+						w_crop = scr->scr_width - wwin->client.x;
+					if (wwin->client.y + wwin->client.height > scr->scr_height)
+						h_crop = scr->scr_height - wwin->client.y;
+
+					pimg = XGetImage(dpy, scr->root_win, x_crop, y_crop,
+							(wwin->client.x > 0)?w_crop:w_crop + wwin->client.x,
+							(wwin->client.y > 0)?h_crop:h_crop + wwin->client.y,
+							AllPlanes, ZPixmap);
+
+					if (pimg) {
+						img = RCreateImageFromXImage(scr->rcontext, pimg, None);
+						XDestroyImage(pimg);
+					}
+				}
+			}
+			break;
+		case PRINT_PARTIAL:
+			pimg = imageCaptureArea(scr);
+			if (pimg) {
+				img = RCreateImageFromXImage(scr->rcontext, pimg, None);
+				XDestroyImage(pimg);
+			}
+			break;
+		default:
+			/* PRINT_SCREEN*/
+			img = RCreateImageFromDrawable(scr->rcontext, scr->root_win, None);
+	}
+
+	if (img) {
+#ifdef USE_PNG
+		if (RSaveTitledImage(img, filepath, (char *)(filetype + 1), "Screenshot from Window Maker")) {
+#else
+		if (RSaveTitledImage(img, filepath, (char *)(filetype + 1), "Screenshot from Window Maker")) {
+#endif
+			scale_img = RSmoothScaleImage(img, scr->scr_width / 10, scr->scr_height / 10);
+			showMiniScreenshot(scr, scale_img);
+			RReleaseImage(scale_img);
+#ifdef DEBUG
+			wmessage("screenshot filepath: %s", filepath);
+#endif
+		}
+		RReleaseImage(img);
+	}
+	wfree(filepath);
+	wfree(screenshot_dir);
+}
