@@ -13,10 +13,9 @@
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
- *  USA.
+ *  You should have received a copy of the GNU General Public License along
+ *  with this program; if not, write to the Free Software Foundation, Inc.,
+ *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #ifndef WMWDEFAULTS_H_
@@ -33,11 +32,12 @@
 #define WA_FOCUSABLE  		(1<<6)
 #define WA_OMNIPRESENT 	 	(1<<7)
 #define WA_SKIP_WINDOW_LIST  	(1<<8)
-#define WA_FLOATING  		(1<<9)
-#define WA_IGNORE_KEYS 		(1<<10)
-#define WA_IGNORE_MOUSE  	(1<<11)
-#define WA_IGNORE_HIDE_OTHERS	(1<<12)
-#define WA_NOT_APPLICATION	(1<<13)
-#define WA_DONT_MOVE_OFF	(1<<14)
+#define WA_SKIP_SWITCHPANEL  	(1<<9)
+#define WA_FLOATING  		(1<<10)
+#define WA_IGNORE_KEYS 		(1<<11)
+#define WA_IGNORE_MOUSE  	(1<<12)
+#define WA_IGNORE_HIDE_OTHERS	(1<<13)
+#define WA_NOT_APPLICATION	(1<<14)
+#define WA_DONT_MOVE_OFF	(1<<15)
 
 #endif
