@@ -48,33 +48,95 @@
 #include "xinerama.h"
 #include "winmenu.h"
 
-#define MC_MAXIMIZE	0
-#define MC_MINIATURIZE	1
-#define MC_SHADE	2
-#define MC_HIDE		3
-#define MC_MOVERESIZE   4
-#define MC_SELECT       5
-#define MC_DUMMY_MOVETO 6
-#define MC_PROPERTIES   7
-#define MC_OPTIONS      8
-#define MC_SHORTCUT     8
-#define MC_RELAUNCH     9
 
-#define MC_CLOSE        10
-#define MC_KILL         11
+static WMenu *makeWorkspaceMenu(WScreen *scr);
+static WMenu *makeOptionsMenu(WScreen *scr);
+static WMenu *makeMaximizeMenu(WScreen *scr);
 
-#define WO_KEEP_ON_TOP		0
-#define WO_KEEP_AT_BOTTOM	1
-#define WO_OMNIPRESENT		2
-#define WO_ENTRIES		3
+/*
+ * Define the Menu entry that will be present in the Window menu
+ *
+ * The order of this list defines the order in which they will appear;
+ * make sure to keep the structure below aligned with the list because
+ * the constant index is used in many places.
+ */
+enum
+{
+	MC_MAXIMIZE,
+	MC_OTHERMAX,
+	MC_MINIATURIZE,
+	MC_SHADE,
+	MC_HIDE,
+	MC_MOVERESIZE,
+	MC_SELECT,
+	MC_CHANGEWKSPC,
+	MC_PROPERTIES,
+	MC_OPTIONS,
+	MC_RELAUNCH,
+	MC_CLOSE,
+	MC_KILL
+};
 
-/**** Global data ***/
-extern Time LastTimestamp;
-extern Atom _XA_WM_DELETE_WINDOW;
-extern Atom _XA_GNUSTEP_WM_MINIATURIZE_WINDOW;
+static const struct {
+	const char *label;
+	WMenu *(*generate_submenu)(WScreen *scr);
+} window_menu_entries[] = {
+	[MC_MAXIMIZE]    = { N_("Maximize"), NULL },
+	[MC_OTHERMAX]    = { N_("Other maximization"), makeMaximizeMenu },
+	[MC_MINIATURIZE] = { N_("Miniaturize"), NULL },
+	[MC_SHADE]       = { N_("Shade"), NULL },
+	[MC_HIDE]        = { N_("Hide"), NULL },
+	[MC_MOVERESIZE]  = { N_("Resize/Move"), NULL },
+	[MC_SELECT]      = { N_("Select"), NULL },
+	[MC_CHANGEWKSPC] = { N_("Move To"), makeWorkspaceMenu },
+	[MC_PROPERTIES]  = { N_("Attributes..."), NULL },
+	[MC_OPTIONS]     = { N_("Options"), makeOptionsMenu },
+	[MC_RELAUNCH]    = { N_("Launch"), NULL },
+	[MC_CLOSE]       = { N_("Close"), NULL },
+	[MC_KILL]        = { N_("Kill"), NULL }
+};
 
-extern WShortKey wKeyBindings[WKBD_LAST];
+/*
+ * Defines the menu entries for the Options sub-menu
+ *
+ * These options will be placed at the beginning of the menu, the rest will
+ * be populated with the Window Shortcut possibilities
+ */
+enum
+{
+	WO_KEEP_ON_TOP,
+	WO_KEEP_AT_BOTTOM,
+	WO_OMNIPRESENT
+};
 
+static const char *const menu_options_entries[] = {
+	[WO_KEEP_ON_TOP]    = N_("Keep on top"),
+	[WO_KEEP_AT_BOTTOM] = N_("Keep at bottom"),
+	[WO_OMNIPRESENT]    = N_("Omnipresent")
+};
+
+/*
+ * Defines the menu entries for the Other maximization sub-menu
+ */
+static const struct {
+	unsigned int indicator;
+	const char *label;
+	unsigned int shortcut_idx;
+	int maxim_direction;
+} menu_maximize_entries[] = {
+	{ MI_SNAP_V, N_("Vertical"), WKBD_VMAXIMIZE, MAX_VERTICAL },
+	{ MI_SNAP_H, N_("Horizontal"), WKBD_HMAXIMIZE, MAX_HORIZONTAL },
+	{ MI_CENTRAL, N_("Central"), WKBD_CENTRAL, MAX_CENTRAL },
+	{ MI_SNAP_LH, N_("Left half"), WKBD_LHMAXIMIZE, MAX_VERTICAL | MAX_LEFTHALF },
+	{ MI_SNAP_RH, N_("Right half"), WKBD_RHMAXIMIZE, MAX_VERTICAL | MAX_RIGHTHALF },
+	{ MI_SNAP_TH, N_("Top half"), WKBD_THMAXIMIZE, MAX_HORIZONTAL | MAX_TOPHALF },
+	{ MI_SNAP_BH, N_("Bottom half"), WKBD_BHMAXIMIZE, MAX_HORIZONTAL | MAX_BOTTOMHALF },
+	{ MI_SNAP_TL, N_("Top left"), WKBD_LTCMAXIMIZE, MAX_LEFTHALF | MAX_TOPHALF },
+	{ MI_SNAP_TR, N_("Top right"), WKBD_RTCMAXIMIZE, MAX_RIGHTHALF | MAX_TOPHALF },
+	{ MI_SNAP_BL, N_("Bottom left"), WKBD_LBCMAXIMIZE, MAX_LEFTHALF | MAX_BOTTOMHALF },
+	{ MI_SNAP_BR, N_("Bottom right"), WKBD_RBCMAXIMIZE, MAX_RIGHTHALF | MAX_BOTTOMHALF },
+	{ MI_SNAP_TILED, N_("Tiled"), WKBD_MAXIMUS, MAX_MAXIMUS }
+};
 
 static void updateOptionsMenu(WMenu * menu, WWindow * wwin);
 
@@ -82,6 +144,9 @@ static void execWindowOptionCommand(WMenu * menu, WMenuEntry * entry)
 {
 	WWindow *wwin = (WWindow *) entry->clientdata;
 
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) menu;
+
 	switch (entry->order) {
 	case WO_KEEP_ON_TOP:
 		if (wwin->frame->core->stacking->window_level != WMFloatingLevel)
@@ -103,6 +168,77 @@ static void execWindowOptionCommand(WMenu * menu, WMenuEntry * entry)
 	}
 }
 
+static void execMaximizeCommand(WMenu * menu, WMenuEntry * entry)
+{
+	WWindow *wwin = (WWindow *) entry->clientdata;
+
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) menu;
+
+	handleMaximize(wwin, menu_maximize_entries[entry->order].maxim_direction);
+}
+
+static void updateUnmaximizeShortcut(WMenuEntry * entry, int flags)
+{
+	int key;
+
+	switch (flags & (MAX_HORIZONTAL | MAX_VERTICAL | MAX_LEFTHALF | MAX_RIGHTHALF | MAX_TOPHALF | MAX_BOTTOMHALF | MAX_MAXIMUS | MAX_CENTRAL)) {
+	case MAX_HORIZONTAL:
+		key = WKBD_HMAXIMIZE;
+		break;
+
+	case MAX_VERTICAL:
+		key = WKBD_VMAXIMIZE;
+		break;
+
+	case MAX_CENTRAL:
+		key = WKBD_CENTRAL;
+		break;
+
+	case MAX_LEFTHALF | MAX_VERTICAL:
+		key = WKBD_LHMAXIMIZE;
+		break;
+
+	case MAX_RIGHTHALF | MAX_VERTICAL:
+		key = WKBD_RHMAXIMIZE;
+		break;
+
+	case MAX_TOPHALF | MAX_HORIZONTAL:
+		key = WKBD_THMAXIMIZE;
+		break;
+
+	case MAX_BOTTOMHALF | MAX_HORIZONTAL:
+		key = WKBD_BHMAXIMIZE;
+		break;
+
+	case MAX_LEFTHALF | MAX_TOPHALF:
+		key = WKBD_LTCMAXIMIZE;
+		break;
+
+	case MAX_RIGHTHALF | MAX_TOPHALF:
+		key = WKBD_RTCMAXIMIZE;
+		break;
+
+	case MAX_LEFTHALF | MAX_BOTTOMHALF:
+		key = WKBD_LBCMAXIMIZE;
+		break;
+
+	case MAX_RIGHTHALF | MAX_BOTTOMHALF:
+		key = WKBD_RBCMAXIMIZE;
+		break;
+
+	case MAX_MAXIMUS:
+		key = WKBD_MAXIMUS;
+		break;
+
+	default:
+		key = WKBD_MAXIMIZE;
+		break;
+	}
+
+	entry->rtext = GetShortcutKey(wKeyBindings[key]);
+}
+
 static void execMenuCommand(WMenu * menu, WMenuEntry * entry)
 {
 	WWindow *wwin = (WWindow *) entry->clientdata;
@@ -113,7 +249,8 @@ static void execMenuCommand(WMenu * menu, WMenuEntry * entry)
 	switch (entry->order) {
 	case MC_CLOSE:
 		/* send delete message */
-		wClientSendProtocol(wwin, _XA_WM_DELETE_WINDOW, LastTimestamp);
+		wClientSendProtocol(wwin, w_global.atom.wm.delete_window,
+								  w_global.timestamp.last_event);
 		break;
 
 	case MC_KILL:
@@ -134,7 +271,8 @@ static void execMenuCommand(WMenu * menu, WMenuEntry * entry)
 			wDeiconifyWindow(wwin);
 		} else {
 			if (wwin->protocols.MINIATURIZE_WINDOW) {
-				wClientSendProtocol(wwin, _XA_GNUSTEP_WM_MINIATURIZE_WINDOW, LastTimestamp);
+				wClientSendProtocol(wwin, w_global.atom.gnustep.wm_miniaturize_window,
+										  w_global.timestamp.last_event);
 			} else {
 				wIconifyWindow(wwin);
 			}
@@ -145,7 +283,8 @@ static void execMenuCommand(WMenu * menu, WMenuEntry * entry)
 		if (wwin->flags.maximized)
 			wUnmaximizeWindow(wwin);
 		else
-			wMaximizeWindow(wwin, MAX_VERTICAL | MAX_HORIZONTAL);
+			wMaximizeWindow(wwin, MAX_VERTICAL | MAX_HORIZONTAL,
+					wGetHeadForWindow(wwin));
 		break;
 
 	case MC_SHADE:
@@ -185,6 +324,9 @@ static void switchWSCommand(WMenu * menu, WMenuEntry * entry)
 {
 	WWindow *wwin = (WWindow *) entry->clientdata;
 
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) menu;
+
 	wSelectWindow(wwin, False);
 	wWindowChangeWorkspace(wwin, entry->order);
 }
@@ -193,7 +335,10 @@ static void makeShortcutCommand(WMenu * menu, WMenuEntry * entry)
 {
 	WWindow *wwin = (WWindow *) entry->clientdata;
 	WScreen *scr = wwin->screen_ptr;
-	int index = entry->order - WO_ENTRIES;
+	int index = entry->order - wlengthof(menu_options_entries);
+
+	/* Parameter not used, but tell the compiler that it is ok */
+	(void) menu;
 
 	if (scr->shortcutWindows[index]) {
 		WMFreeArray(scr->shortcutWindows[index]);
@@ -258,25 +403,20 @@ static void updateWorkspaceMenu(WMenu * menu)
 
 static void updateMakeShortcutMenu(WMenu * menu, WWindow * wwin)
 {
-	WMenu *smenu = menu->cascades[menu->entries[MC_SHORTCUT]->cascade];
+	WMenu *smenu = menu->cascades[menu->entries[MC_OPTIONS]->cascade];
 	int i;
-	char *buffer;
-	int buflen;
+	char buffer[64];
 	KeyCode kcode;
 
 	if (!smenu)
 		return;
 
-	buflen = strlen(_("Set Shortcut")) + 16;
-	buffer = wmalloc(buflen);
-
-	for (i = WO_ENTRIES; i < smenu->entry_no; i++) {
-		char *tmp;
-		int shortcutNo = i - WO_ENTRIES;
+	for (i = wlengthof(menu_options_entries); i < smenu->entry_no; i++) {
+		int shortcutNo = i - wlengthof(menu_options_entries);
 		WMenuEntry *entry = smenu->entries[i];
 		WMArray *shortSelWindows = wwin->screen_ptr->shortcutWindows[shortcutNo];
 
-		snprintf(buffer, buflen, "%s %i", _("Set Shortcut"), shortcutNo + 1);
+		snprintf(buffer, sizeof(buffer), _("Set Shortcut %i"), shortcutNo + 1);
 
 		if (!shortSelWindows) {
 			entry->flags.indicator_on = 0;
@@ -297,12 +437,28 @@ static void updateMakeShortcutMenu(WMenu * menu, WWindow * wwin)
 		kcode = wKeyBindings[WKBD_WINDOW1 + shortcutNo].keycode;
 
 		if (kcode) {
-			if ((tmp = GetShortcutKey(wKeyBindings[WKBD_WINDOW1 + shortcutNo]))
-			    && (!entry->rtext || strcmp(tmp, entry->rtext) != 0)) {
-				if (entry->rtext)
+			char *tmp;
+
+			tmp = GetShortcutKey(wKeyBindings[WKBD_WINDOW1 + shortcutNo]);
+			if (tmp == NULL) {
+				if (entry->rtext != NULL) {
+					/* There was a shortcut, but there is no more */
 					wfree(entry->rtext);
+					entry->rtext = NULL;
+					smenu->flags.realized = 0;
+				}
+			} else if (entry->rtext == NULL) {
+				/* There was no shortcut, but there is one now */
 				entry->rtext = tmp;
 				smenu->flags.realized = 0;
+			} else if (strcmp(tmp, entry->rtext) != 0) {
+				/* There was a shortcut, but it has changed */
+				wfree(entry->rtext);
+				entry->rtext = tmp;
+				smenu->flags.realized = 0;
+			} else {
+				/* There was a shortcut but it did not change */
+				wfree(tmp);
 			}
 			wMenuSetEnabled(smenu, i, True);
 		} else {
@@ -315,7 +471,6 @@ static void updateMakeShortcutMenu(WMenu * menu, WWindow * wwin)
 		}
 		entry->clientdata = wwin;
 	}
-	wfree(buffer);
 	if (!smenu->flags.realized)
 		wMenuRealize(smenu);
 }
@@ -329,16 +484,33 @@ static void updateOptionsMenu(WMenu * menu, WWindow * wwin)
 	smenu->entries[WO_KEEP_ON_TOP]->flags.indicator_on =
 	    (wwin->frame->core->stacking->window_level == WMFloatingLevel) ? 1 : 0;
 	wMenuSetEnabled(smenu, WO_KEEP_ON_TOP, !wwin->flags.miniaturized);
+	smenu->entries[WO_KEEP_ON_TOP]->rtext = GetShortcutKey(wKeyBindings[WKBD_KEEP_ON_TOP]);
 
 	/* keep at bottom check */
 	smenu->entries[WO_KEEP_AT_BOTTOM]->clientdata = wwin;
 	smenu->entries[WO_KEEP_AT_BOTTOM]->flags.indicator_on =
 	    (wwin->frame->core->stacking->window_level == WMSunkenLevel) ? 1 : 0;
 	wMenuSetEnabled(smenu, WO_KEEP_AT_BOTTOM, !wwin->flags.miniaturized);
+	smenu->entries[WO_KEEP_AT_BOTTOM]->rtext = GetShortcutKey(wKeyBindings[WKBD_KEEP_AT_BOTTOM]);
 
 	/* omnipresent check */
 	smenu->entries[WO_OMNIPRESENT]->clientdata = wwin;
 	smenu->entries[WO_OMNIPRESENT]->flags.indicator_on = IS_OMNIPRESENT(wwin);
+	smenu->entries[WO_OMNIPRESENT]->rtext = GetShortcutKey(wKeyBindings[WKBD_OMNIPRESENT]);
+
+	smenu->flags.realized = 0;
+	wMenuRealize(smenu);
+}
+
+static void updateMaximizeMenu(WMenu * menu, WWindow * wwin)
+{
+	WMenu *smenu = menu->cascades[menu->entries[MC_OTHERMAX]->cascade];
+	int i;
+
+	for (i = 0; i < smenu->entry_no; i++) {
+		smenu->entries[i]->clientdata = wwin;
+		smenu->entries[i]->rtext = GetShortcutKey(wKeyBindings[menu_maximize_entries[i].shortcut_idx]);
+	}
 
 	smenu->flags.realized = 0;
 	wMenuRealize(smenu);
@@ -356,29 +528,13 @@ static WMenu *makeWorkspaceMenu(WScreen * scr)
 
 	updateWorkspaceMenu(menu);
 
-	return menu;
-}
-
-static WMenu *makeMakeShortcutMenu(WScreen * scr, WMenu * menu)
-{
 	/*
-	   WMenu *menu;
+	 * The Workspace Menu is made visible in the screen structure because
+	 * it is updated when there is a change on workspaces. This was done
+	 * to be efficient, avoiding re-generating completely the window menu
+	 * and its sub-menus every time it is needed.
 	 */
-	int i;
-	/*
-	   menu = wMenuCreate(scr, NULL, False);
-	   if (!menu) {
-	   wwarning(_("could not create submenu for window menu"));
-	   return NULL;
-	   }
-	 */
-
-	for (i = 0; i < MAX_WINDOW_SHORTCUTS; i++) {
-		WMenuEntry *entry;
-		entry = wMenuAddCallback(menu, "", makeShortcutCommand, NULL);
-
-		entry->flags.indicator = 1;
-	}
+	scr->workspace_submenu = menu;
 
 	return menu;
 }
@@ -387,6 +543,7 @@ static WMenu *makeOptionsMenu(WScreen * scr)
 {
 	WMenu *menu;
 	WMenuEntry *entry;
+	int i;
 
 	menu = wMenuCreate(scr, NULL, False);
 	if (!menu) {
@@ -394,17 +551,37 @@ static WMenu *makeOptionsMenu(WScreen * scr)
 		return NULL;
 	}
 
-	entry = wMenuAddCallback(menu, _("Keep on top"), execWindowOptionCommand, NULL);
-	entry->flags.indicator = 1;
-	entry->flags.indicator_type = MI_CHECK;
+	for (i = 0; i < wlengthof(menu_options_entries); i++) {
+		entry = wMenuAddCallback(menu, _(menu_options_entries[i]), execWindowOptionCommand, NULL);
+		entry->flags.indicator = 1;
+		entry->flags.indicator_type = MI_CHECK;
+	}
 
-	entry = wMenuAddCallback(menu, _("Keep at bottom"), execWindowOptionCommand, NULL);
-	entry->flags.indicator = 1;
-	entry->flags.indicator_type = MI_CHECK;
+	for (i = 0; i < MAX_WINDOW_SHORTCUTS; i++) {
+		entry = wMenuAddCallback(menu, "", makeShortcutCommand, NULL);
+		entry->flags.indicator = 1;
+	}
 
-	entry = wMenuAddCallback(menu, _("Omnipresent"), execWindowOptionCommand, NULL);
-	entry->flags.indicator = 1;
-	entry->flags.indicator_type = MI_CHECK;
+	return menu;
+}
+
+static WMenu *makeMaximizeMenu(WScreen * scr)
+{
+	WMenu *menu;
+	WMenuEntry *entry;
+	int i;
+
+	menu = wMenuCreate(scr, NULL, False);
+	if (!menu) {
+		wwarning(_("could not create submenu for window menu"));
+		return NULL;
+	}
+
+	for (i = 0; i < wlengthof(menu_maximize_entries); i++) {
+		entry = wMenuAddCallback(menu, _(menu_maximize_entries[i].label), execMaximizeCommand, NULL);
+		entry->flags.indicator = entry->flags.indicator_on = 1;
+		entry->flags.indicator_type = menu_maximize_entries[i].indicator;
+	}
 
 	return menu;
 }
@@ -412,54 +589,23 @@ static WMenu *makeOptionsMenu(WScreen * scr)
 static WMenu *createWindowMenu(WScreen * scr)
 {
 	WMenu *menu;
-	WMenuEntry *entry;
+	int i;
 
 	menu = wMenuCreate(scr, NULL, False);
-	/*
-	 * Warning: If you make some change that affects the order of the
-	 * entries, you must update the command #defines in the top of
-	 * this file.
-	 */
-	entry = wMenuAddCallback(menu, _("Maximize"), execMenuCommand, NULL);
-	entry->rtext = GetShortcutKey(wKeyBindings[WKBD_MAXIMIZE]);
 
-	entry = wMenuAddCallback(menu, _("Miniaturize"), execMenuCommand, NULL);
-	entry->rtext = GetShortcutKey(wKeyBindings[WKBD_MINIATURIZE]);
+	for (i = 0; i < wlengthof(window_menu_entries); i++) {
+		WMenuEntry *entry;
 
-	entry = wMenuAddCallback(menu, _("Shade"), execMenuCommand, NULL);
-	entry->rtext = GetShortcutKey(wKeyBindings[WKBD_SHADE]);
+		entry = wMenuAddCallback(menu, _(window_menu_entries[i].label),
+		                         (window_menu_entries[i].generate_submenu == NULL)?execMenuCommand:NULL,
+		                         NULL);
+		if (window_menu_entries[i].generate_submenu != NULL) {
+			WMenu *submenu;
 
-	entry = wMenuAddCallback(menu, _("Hide"), execMenuCommand, NULL);
-	entry->rtext = GetShortcutKey(wKeyBindings[WKBD_HIDE]);
-
-	entry = wMenuAddCallback(menu, _("Resize/Move"), execMenuCommand, NULL);
-	entry->rtext = GetShortcutKey(wKeyBindings[WKBD_MOVERESIZE]);
-
-	entry = wMenuAddCallback(menu, _("Select"), execMenuCommand, NULL);
-	entry->rtext = GetShortcutKey(wKeyBindings[WKBD_SELECT]);
-
-	entry = wMenuAddCallback(menu, _("Move To"), NULL, NULL);
-	scr->workspace_submenu = makeWorkspaceMenu(scr);
-	if (scr->workspace_submenu)
-		wMenuEntrySetCascade(menu, entry, scr->workspace_submenu);
-
-	entry = wMenuAddCallback(menu, _("Attributes..."), execMenuCommand, NULL);
-
-	entry = wMenuAddCallback(menu, _("Options"), NULL, NULL);
-	wMenuEntrySetCascade(menu, entry, makeMakeShortcutMenu(scr, makeOptionsMenu(scr)));
-
-	/*
-	   entry = wMenuAddCallback(menu, _("Select Shortcut"), NULL, NULL);
-	   wMenuEntrySetCascade(menu, entry, makeMakeShortcutMenu(scr));
-	 */
-
-	entry = wMenuAddCallback(menu, _("Launch"), execMenuCommand, NULL);
-	entry->rtext = GetShortcutKey(wKeyBindings[WKBD_RELAUNCH]);
-
-	entry = wMenuAddCallback(menu, _("Close"), execMenuCommand, NULL);
-	entry->rtext = GetShortcutKey(wKeyBindings[WKBD_CLOSE]);
-
-	entry = wMenuAddCallback(menu, _("Kill"), execMenuCommand, NULL);
+			submenu = window_menu_entries[i].generate_submenu(scr);
+			wMenuEntrySetCascade(menu, entry, submenu);
+		}
+	}
 
 	return menu;
 }
@@ -486,6 +632,7 @@ static void updateMenuForWindow(WMenu * menu, WWindow * wwin)
 	int i;
 
 	updateOptionsMenu(menu, wwin);
+	updateMaximizeMenu(menu, wwin);
 
 	updateMakeShortcutMenu(menu, wwin);
 
@@ -515,12 +662,14 @@ static void updateMenuForWindow(WMenu * menu, WWindow * wwin)
 			text = _("Unmaximize");
 
 		menu->entries[MC_MAXIMIZE]->text = text;
+		updateUnmaximizeShortcut(menu->entries[MC_MAXIMIZE], wwin->flags.maximized);
 	} else {
 		static char *text = NULL;
 		if (!text)
 			text = _("Maximize");
 
 		menu->entries[MC_MAXIMIZE]->text = text;
+		menu->entries[MC_MAXIMIZE]->rtext = GetShortcutKey(wKeyBindings[WKBD_MAXIMIZE]);
 	}
 	wMenuSetEnabled(menu, MC_MAXIMIZE, IS_RESIZABLE(wwin));
 
@@ -558,7 +707,7 @@ static void updateMenuForWindow(WMenu * menu, WWindow * wwin)
 		menu->entries[MC_SELECT]->text = text;
 	}
 
-	wMenuSetEnabled(menu, MC_DUMMY_MOVETO, !IS_OMNIPRESENT(wwin));
+	wMenuSetEnabled(menu, MC_CHANGEWKSPC, !IS_OMNIPRESENT(wwin));
 
 	if (!wwin->flags.inspector_open) {
 		wMenuSetEnabled(menu, MC_PROPERTIES, True);
@@ -566,6 +715,17 @@ static void updateMenuForWindow(WMenu * menu, WWindow * wwin)
 		wMenuSetEnabled(menu, MC_PROPERTIES, False);
 	}
 
+	/* Update shortcut labels except for (Un)Maximize which is
+	 * handled separately.
+	 */
+	menu->entries[MC_MINIATURIZE]->rtext = GetShortcutKey(wKeyBindings[WKBD_MINIATURIZE]);
+	menu->entries[MC_SHADE]->rtext = GetShortcutKey(wKeyBindings[WKBD_SHADE]);
+	menu->entries[MC_HIDE]->rtext = GetShortcutKey(wKeyBindings[WKBD_HIDE]);
+	menu->entries[MC_MOVERESIZE]->rtext = GetShortcutKey(wKeyBindings[WKBD_MOVERESIZE]);
+	menu->entries[MC_SELECT]->rtext = GetShortcutKey(wKeyBindings[WKBD_SELECT]);
+	menu->entries[MC_RELAUNCH]->rtext = GetShortcutKey(wKeyBindings[WKBD_RELAUNCH]);
+	menu->entries[MC_CLOSE]->rtext = GetShortcutKey(wKeyBindings[WKBD_CLOSE]);
+
 	/* set the client data of the entries to the window */
 	for (i = 0; i < menu->entry_no; i++) {
 		menu->entries[i]->clientdata = wwin;
@@ -573,18 +733,17 @@ static void updateMenuForWindow(WMenu * menu, WWindow * wwin)
 
 	for (i = 0; i < scr->workspace_submenu->entry_no; i++) {
 		scr->workspace_submenu->entries[i]->clientdata = wwin;
-		if (i == scr->current_workspace) {
+		if (i == scr->current_workspace)
 			wMenuSetEnabled(scr->workspace_submenu, i, False);
-		} else {
+		else
 			wMenuSetEnabled(scr->workspace_submenu, i, True);
-		}
 	}
 
 	menu->flags.realized = 0;
 	wMenuRealize(menu);
 }
 
-static WMenu *open_window_menu_core(WWindow *wwin, int x, int y)
+static WMenu *open_window_menu_core(WWindow *wwin)
 {
 	WScreen *scr = wwin->screen_ptr;
 	WMenu *menu;
@@ -615,23 +774,23 @@ static WMenu *open_window_menu_core(WWindow *wwin, int x, int y)
 	return menu;
 }
 
-static void prepare_menu_position(WMenu *menu, int x, int y)
+static void prepare_menu_position(WMenu *menu, int *x, int *y)
 {
 	WMRect rect;
 
 	rect = wGetRectForHead(menu->frame->screen_ptr,
 			       wGetHeadForPointerLocation(menu->frame->screen_ptr));
-	if (x < rect.pos.x - menu->frame->core->width / 2)
-		x = rect.pos.x - menu->frame->core->width / 2;
-	if (y < rect.pos.y)
-		y = rect.pos.y;
+	if (*x < rect.pos.x - menu->frame->core->width / 2)
+		*x = rect.pos.x - menu->frame->core->width / 2;
+	if (*y < rect.pos.y)
+		*y = rect.pos.y;
 }
 
 void OpenWindowMenu(WWindow *wwin, int x, int y, int keyboard)
 {
 	WMenu *menu;
 
-	menu = open_window_menu_core(wwin, x, y);
+	menu = open_window_menu_core(wwin);
 	if (!menu)
 		return;
 
@@ -643,7 +802,7 @@ void OpenWindowMenu(WWindow *wwin, int x, int y, int keyboard)
 		x = wwin->frame_x;
 
 	/* Common menu position */
-	prepare_menu_position(menu, x, y);
+	prepare_menu_position(menu, &x, &y);
 
 	if (!wwin->flags.internal_window)
 		wMenuMapAt(menu, x, y, keyboard);
@@ -655,7 +814,7 @@ void OpenWindowMenu2(WWindow *wwin, int x, int y, int keyboard)
 	WMenu *menu;
 	WScreen *scr = wwin->screen_ptr;
 
-	menu = open_window_menu_core(wwin, x, y);
+	menu = open_window_menu_core(wwin);
 	if (!menu)
 		return;
 
@@ -668,7 +827,7 @@ void OpenWindowMenu2(WWindow *wwin, int x, int y, int keyboard)
 	x -= menu->frame->core->width / 2;
 
 	/* Common menu position */
-	prepare_menu_position(menu, x, y);
+	prepare_menu_position(menu, &x, &y);
 
 	if (!wwin->flags.internal_window)
 		wMenuMapAt(menu, x, y, keyboard);
@@ -678,7 +837,7 @@ void OpenMiniwindowMenu(WWindow * wwin, int x, int y)
 {
 	WMenu *menu;
 
-	menu = open_window_menu_core(wwin, x, y);
+	menu = open_window_menu_core(wwin);
 	if (!menu)
 		return;
 
