@@ -18,6 +18,8 @@
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
+#include <stdlib.h>
+
 #include "wconfig.h"
 
 #include "xinerama.h"
@@ -26,8 +28,9 @@
 #include "window.h"
 #include "framewin.h"
 #include "placement.h"
+#include "dock.h"
 
-#ifdef XINERAMA
+#ifdef USE_XINERAMA
 # ifdef SOLARIS_XINERAMA	/* sucks */
 #  include <X11/extensions/xinerama.h>
 # else
@@ -40,7 +43,7 @@ void wInitXinerama(WScreen * scr)
 	scr->xine_info.primary_head = 0;
 	scr->xine_info.screens = NULL;
 	scr->xine_info.count = 0;
-#ifdef XINERAMA
+#ifdef USE_XINERAMA
 # ifdef SOLARIS_XINERAMA
 	if (XineramaGetState(dpy, scr->screen)) {
 		WXineramaInfo *info = &scr->xine_info;
@@ -79,7 +82,7 @@ void wInitXinerama(WScreen * scr)
 		XFree(xine_screens);
 	}
 # endif				/* !SOLARIS_XINERAMA */
-#endif				/* XINERAMA */
+#endif				/* USE_XINERAMA */
 }
 
 int wGetRectPlacementInfo(WScreen * scr, WMRect rect, int *flags)
@@ -235,6 +238,74 @@ int wGetHeadForWindow(WWindow * wwin)
 	return wGetHeadForRect(wwin->screen_ptr, rect);
 }
 
+/* Find head on left, right, up or down direction relative to current
+head. If there is no screen available on pointed direction, -1 will be
+returned.*/
+int wGetHeadRelativeToCurrentHead(WScreen *scr, int current_head, int direction)
+{
+	short int found = 0;
+	int i;
+	int distance = 0;
+	int smallest_distance = 0;
+	int nearest_head = scr->xine_info.primary_head;
+	WMRect crect = wGetRectForHead(scr, current_head);
+
+	for (i = 0; i < scr->xine_info.count; i++) {
+		if (i == current_head)
+			continue;
+
+		WMRect *rect = &scr->xine_info.screens[i];
+
+		/* calculate distance from the next screen to current one */
+		switch (direction) {
+			case DIRECTION_LEFT:
+				if (rect->pos.x < crect.pos.x) {
+					found = 1;
+					distance = abs((rect->pos.x + (int)rect->size.width)
+							- crect.pos.x) + abs(rect->pos.y + crect.pos.y);
+				}
+				break;
+			case DIRECTION_RIGHT:
+				if (rect->pos.x > crect.pos.x) {
+					found = 1;
+					distance = abs((crect.pos.x + (int)crect.size.width)
+							- rect->pos.x) + abs(rect->pos.y + crect.pos.y);
+				}
+				break;
+			case DIRECTION_UP:
+				if (rect->pos.y < crect.pos.y) {
+					found = 1;
+					distance = abs((rect->pos.y + (int)rect->size.height)
+							- crect.pos.y) + abs(rect->pos.x + crect.pos.x);
+				}
+				break;
+			case DIRECTION_DOWN:
+				if (rect->pos.y > crect.pos.y) {
+					found = 1;
+					distance = abs((crect.pos.y + (int)crect.size.height)
+							- rect->pos.y) + abs(rect->pos.x + crect.pos.x);
+				}
+				break;
+		}
+
+		if (found && distance == 0)
+			return i;
+
+		if (smallest_distance == 0)
+			smallest_distance = distance;
+
+		if (abs(distance) <= smallest_distance) {
+			smallest_distance = distance;
+			nearest_head = i;
+		}
+	}
+
+	if (found && smallest_distance != 0 && nearest_head != current_head)
+		return nearest_head;
+
+	return -1;
+}
+
 int wGetHeadForPoint(WScreen * scr, WMPoint point)
 {
 	int i;
@@ -303,6 +374,29 @@ WArea wGetUsableAreaForHead(WScreen * scr, int head, WArea * totalAreaPtr, Bool
 	} else
 		usableArea = totalArea;
 
+	if (noicons) {
+		/* check if user wants dock covered */
+		if (scr->dock && wPreferences.no_window_over_dock && wAppIconTouchesHead(scr->dock->icon_array[0], head)) {
+			int offset = wPreferences.icon_size + DOCK_EXTRA_SPACE;
+
+			if (scr->dock->on_right_side)
+				usableArea.x2 -= offset;
+			else
+				usableArea.x1 += offset;
+		}
+
+		/* check if icons are on the same side as dock, and adjust if not done already */
+		if (scr->dock && wPreferences.no_window_over_icons && !wPreferences.no_window_over_dock && (wPreferences.icon_yard & IY_VERT)) {
+			int offset = wPreferences.icon_size + DOCK_EXTRA_SPACE;
+
+			if (scr->dock->on_right_side && (wPreferences.icon_yard & IY_RIGHT))
+				usableArea.x2 -= offset;
+			/* can't use IY_LEFT in if, it's 0 ... */
+			if (!scr->dock->on_right_side && !(wPreferences.icon_yard & IY_RIGHT))
+				usableArea.x1 += offset;
+		}
+	}
+
 	return usableArea;
 }
 
@@ -316,3 +410,33 @@ WMPoint wGetPointToCenterRectInHead(WScreen * scr, int head, int width, int heig
 
 	return p;
 }
+
+/* Find the bounding rect of the union of two rectangles */
+void wGetRectUnion(const WMRect *rect1, const WMRect *rect2, WMRect *dest)
+{
+	int dest_x, dest_y;
+	int dest_w, dest_h;
+
+	dest_x = rect1->pos.x;
+	dest_y = rect1->pos.y;
+	dest_w = rect1->size.width;
+	dest_h = rect1->size.height;
+
+	if (rect2->pos.x < dest_x) {
+		dest_w += dest_x - rect2->pos.x;
+		dest_x = rect2->pos.x;
+	}
+	if (rect2->pos.y < dest_y) {
+		dest_h += dest_y - rect2->pos.y;
+		dest_y = rect2->pos.y;
+	}
+	if (rect2->pos.x + rect2->size.width > dest_x + dest_w)
+		dest_w = rect2->pos.x + rect2->size.width - dest_x;
+	if (rect2->pos.y + rect2->size.height > dest_y + dest_h)
+		dest_h = rect2->pos.y + rect2->size.height - dest_y;
+
+	dest->pos.x = dest_x;
+	dest->pos.y = dest_y;
+	dest->size.width = dest_w;
+	dest->size.height = dest_h;
+}
