@@ -39,6 +39,7 @@
 
 #include "wraster.h"
 #include "scale.h"
+#include "wr_i18n.h"
 
 
 #ifndef HAVE_FLOAT_MATHFUNC
@@ -406,7 +407,8 @@ static Bool setupPseudoColorColormap(RContext * context)
 			}
 
 			if (theMap < 0) {
-				puts("wrlib: no std cmap found");
+				fprintf(stderr, _("wrlib: no standard colormap found for visual 0x%lX\n"),
+				        context->visual->visualid);
 			}
 
 			if (theMap >= 0 && allocateStandardPseudoColor(context, &maps[theMap])) {
@@ -453,7 +455,7 @@ static void gatherconfig(RContext * context, int screen_n)
 	if (ptr) {
 		float g1, g2, g3;
 		if (sscanf(ptr, "%f/%f/%f", &g1, &g2, &g3) != 3 || g1 <= 0.0F || g2 <= 0.0F || g3 <= 0.0F) {
-			printf("wrlib: invalid value(s) for gamma correction \"%s\"\n", ptr);
+			fprintf(stderr, _("wrlib: invalid value \"%s\" for %s\n"), ptr, "WRASTER_GAMMA");
 		} else {
 			context->attribs->flags |= RC_GammaCorrection;
 			context->attribs->rgamma = g1;
@@ -465,7 +467,7 @@ static void gatherconfig(RContext * context, int screen_n)
 	if (ptr) {
 		int i;
 		if (sscanf(ptr, "%d", &i) != 1 || i < 2 || i > 6) {
-			printf("wrlib: invalid value for color resolution \"%s\"\n", ptr);
+			fprintf(stderr, _("wrlib: invalid value \"%s\" for %s\n"), ptr, "WRASTER_COLOR_RESOLUTION");
 		} else {
 			context->attribs->flags |= RC_ColorsPerChannel;
 			context->attribs->colors_per_channel = i;
