@@ -3,6 +3,7 @@
  * Raster graphics library
  *
  * Copyright (c) 1998-2003 Alfredo K. Kojima
+ * Copyright (c) 2013-2023 Window Maker Team
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -16,7 +17,8 @@
  *
  *  You should have received a copy of the GNU Library General Public
  *  License along with this library; if not, write to the Free
- *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
+ *  MA 02110-1301, USA.
  */
 
 #include <config.h>
@@ -28,17 +30,32 @@
 #include <fcntl.h>
 #include <sys/stat.h>
 #include <string.h>
-#include <time.h>
 
 #include "wraster.h"
+#include "imgformat.h"
+#include "wr_i18n.h"
 
-extern Bool RSaveXPM(RImage * image, char *filename);
-
-Bool RSaveImage(RImage * image, char *filename, char *format)
+Bool RSaveImage(RImage *image, const char *filename, const char *format)
 {
-	if (strcmp(format, "XPM") != 0) {
-		RErrorCode = RERR_BADFORMAT;
-		return False;
-	}
-	return RSaveXPM(image, filename);
+	return RSaveTitledImage(image, filename, format, NULL);
+}
+
+Bool RSaveTitledImage(RImage *image, const char *filename, const char *format, char *title)
+{
+#ifdef USE_PNG
+	if (strcasecmp(format, "PNG") == 0)
+		return RSavePNG(image, filename, title);
+#endif
+#ifdef USE_JPEG
+	if (strcasecmp(format, "JPG") == 0)
+		return RSaveJPEG(image, filename, title);
+
+	if (strcasecmp(format, "JPEG") == 0)
+		return RSaveJPEG(image, filename, title);
+#endif
+	if (strcasecmp(format, "XPM") == 0)
+		return RSaveXPM(image, filename);
+
+	RErrorCode = RERR_BADFORMAT;
+	return False;
 }
